@@ -16,15 +16,20 @@ const Bookmarks = Module("bookmarks", {
         const historyService   = services.get("history");
         const tagging          = PlacesUtils.tagging;
 
-        this.getFavicon = getFavicon;
-        function getFavicon(uri) {
-            try {
-                return faviconService.getFaviconImageForPage(util.newURI(uri)).spec;
-            }
-            catch (e) {
-                return "";
-            }
+        function getFavicon (uri) {
+            if (typeof uri === "string")
+                uri = util.newURI(uri);
+
+            var url = null;
+            faviconService.getFaviconDataForPage(uri, function (iconURI) {
+                url = iconURI ? iconURI.spec : "";
+            });
+            while (url === null)
+                liberator.threadYield(false, true);
+
+            return url;
         }
+        this.getFavicon = getFavicon;
 
         // Fix for strange Firefox bug:
         // Error: [Exception... "Component returned failure code: 0x8000ffff (NS_ERROR_UNEXPECTED) [nsIObserverService.addObserver]"
@@ -55,9 +60,9 @@ const Bookmarks = Module("bookmarks", {
             this.__defineGetter__("bookmarks", function () this.load());
 
             this.__defineGetter__("keywords",
-                function () [Keyword(k.keyword, k.title, k.icon, k.url) for ([, k] in Iterator(self.bookmarks)) if (k.keyword)]);
+                function () [Keyword(k.keyword, k.title, k.icon, k.url) for (k of self.bookmarks) if (k.keyword)]);
 
-            this.__iterator__ = function () (val for ([, val] in Iterator(self.bookmarks)));
+            this.__iterator__ = function () (val for (val of self.bookmarks));
 
             function loadBookmark(node) {
                 if (node.uri == null) // How does this happen?
@@ -65,7 +70,7 @@ const Bookmarks = Module("bookmarks", {
                 let uri = util.newURI(node.uri);
                 let keyword = bookmarksService.getKeywordForBookmark(node.itemId);
                 let tags = tagging.getTagsForURI(uri, {}) || [];
-                let bmark = Bookmark(node.uri, node.title, node.icon && node.icon.spec, keyword, tags, node.itemId);
+                let bmark = Bookmark(node.uri, node.title, node.icon, keyword, tags, node.itemId);
 
                 bookmarks.push(bmark);
                 return bmark;
@@ -95,16 +100,6 @@ const Bookmarks = Module("bookmarks", {
 
             this.isBookmark = function (id) rootFolders.indexOf(self.findRoot(id)) >= 0;
 
-            this.isRegularBookmark = function findRoot(id) {
-                do {
-                    var root = id;
-                    if (services.get("livemark") && services.get("livemark").isLivemark(id))
-                        return false;
-                    id = bookmarksService.getFolderIdForItem(id);
-                } while (id != bookmarksService.placesRoot && id != root);
-                return rootFolders.indexOf(root) >= 0;
-            };
-
             // since we don't use a threaded bookmark loading (by set preload)
             // anymore, is this loading synchronization still needed? --mst
             let loading = false;
@@ -156,14 +151,14 @@ const Bookmarks = Module("bookmarks", {
                         if (self.isBookmark(itemId)) {
                             let bmark = loadBookmark(readBookmark(itemId));
                             storage.fireEvent(name, "add", bmark);
-                            statusline.updateBookmark();
+                            statusline.updateField("bookmark");
                         }
                     }
                 },
                 onItemRemoved: function onItemRemoved(itemId, folder, index) {
                     if (deleteBookmark(itemId)) {
                         storage.fireEvent(name, "remove", itemId);
-                        statusline.updateBookmark();
+                        statusline.updateField("bookmark");
                     }
                 },
                 onItemChanged: function onItemChanged(itemId, property, isAnnotation, value) {
@@ -201,7 +196,8 @@ const Bookmarks = Module("bookmarks", {
     get format() ({
         anchored: false,
         title: ["URL", "Info"],
-        keys: { text: "url", description: "title", icon: "icon", extra: "extra", tags: "tags", keyword: "keyword" },
+        keys: { text: "url", description: "title", icon: function(item) item.icon || DEFAULT_FAVICON,
+                extra: "extra", tags: "tags", keyword: "keyword" },
         process: [template.icon, template.bookmarkDescription]
     }),
 
@@ -211,7 +207,8 @@ const Bookmarks = Module("bookmarks", {
     },
 
     // if starOnly = true it is saved in the unfiledBookmarksFolder, otherwise in the bookmarksMenuFolder
-    add: function add(starOnly, title, url, keyword, tags, force) {
+    add: function add(starOnly, title, url, keyword, tags, folder, force) {
+        var bs = services.get("bookmarks");
         try {
             let uri = util.createURI(url);
             if (!force) {
@@ -219,20 +216,37 @@ const Bookmarks = Module("bookmarks", {
                     if (bmark[0] == uri.spec) {
                         var id = bmark[5];
                         if (title)
-                            services.get("bookmarks").setItemTitle(id, title);
+                            bs.setItemTitle(id, title);
                         break;
                     }
                 }
             }
+            let folderId = bs[starOnly ? "unfiledBookmarksFolder" : "bookmarksMenuFolder"];
+            if (folder) {
+                let folders = folder.split("/");
+                if (!folders[folders.length - 1])
+                    folders.pop();
+
+                let rootFolderId = folderId;
+                if (folders[0] === "TOOLBAR") {
+                    rootFolderId = bs.toolbarFolder;
+                    folders.shift();
+                }
+                let node = Bookmarks.getBookmarkFolder(folders, rootFolderId);
+                if (node)
+                    folderId = node.itemId;
+                else {
+                    liberator.echoerr("Bookmark folder is not found: `" + folder + "'");
+                    return false;
+                }
+            }
             if (id == undefined)
-                id = services.get("bookmarks").insertBookmark(
-                         services.get("bookmarks")[starOnly ? "unfiledBookmarksFolder" : "bookmarksMenuFolder"],
-                         uri, -1, title || url);
+                id = bs.insertBookmark(folderId, uri, -1, title || url);
             if (!id)
                 return false;
 
             if (keyword)
-                services.get("bookmarks").setKeywordForBookmark(id, keyword);
+                bs.setKeywordForBookmark(id, keyword);
             if (tags) {
                 PlacesUtils.tagging.untagURI(uri, null);
                 PlacesUtils.tagging.tagURI(uri, tags);
@@ -263,21 +277,13 @@ const Bookmarks = Module("bookmarks", {
         }
     },
 
-    isBookmarked: function isBookmarked(url) {
-        try {
-            return services.get("bookmarks").getBookmarkIdsForURI(makeURI(url), {})
-                                   .some(this._cache.isRegularBookmark);
-        }
-        catch (e) {
-            return false;
-        }
-    },
+    isBookmarked: function isBookmarked(url) services.get("bookmarks").isBookmarked(util.newURI(url)),
 
     // returns number of deleted bookmarks
     remove: function remove(url) {
         try {
             let uri = util.newURI(url);
-            let bmarks = services.get("bookmarks").getBookmarkIdsForURI(uri, {}).filter(this._cache.isRegularBookmark);
+            let bmarks = services.get("bookmarks").getBookmarkIdsForURI(uri, {});
             bmarks.forEach(services.get("bookmarks").removeItem);
             return bmarks.length;
         }
@@ -291,7 +297,7 @@ const Bookmarks = Module("bookmarks", {
     // also ensures that each search engine has a Liberator-friendly alias
     getSearchEngines: function getSearchEngines() {
         let searchEngines = [];
-        for (let [, engine] in Iterator(services.get("browserSearch").getVisibleEngines({}))) {
+        for (let [, engine] in Iterator(services.get("search").getVisibleEngines({}))) {
             let alias = engine.alias;
             if (!alias || !/^[a-z0-9_-]+$/.test(alias))
                 alias = engine.name.replace(/^\W*([a-zA-Z_-]+).*/, "$1").toLowerCase();
@@ -319,7 +325,7 @@ const Bookmarks = Module("bookmarks", {
     getSuggestions: function getSuggestions(engineName, query, callback) {
         const responseType = "application/x-suggestions+json";
 
-        let engine = services.get("browserSearch").getEngineByAlias(engineName);
+        let engine = services.get("search").getEngineByAlias(engineName);
         if (engine && engine.supportsResponseType(responseType))
             var queryURI = engine.getSubmission(query, responseType).uri.spec;
         if (!queryURI)
@@ -371,7 +377,7 @@ const Bookmarks = Module("bookmarks", {
                 param = url.substr(offset + 1);
             }
 
-            var engine = services.get("browserSearch").getEngineByAlias(keyword);
+            var engine = services.get("search").getEngineByAlias(keyword);
             if (engine) {
                 var submission = engine.getSubmission(param, null);
                 return [submission.uri.spec, submission.postData];
@@ -443,15 +449,66 @@ const Bookmarks = Module("bookmarks", {
         return null;
     }
 }, {
+    getBookmarkFolder: function (folders, rootFolderId) {
+        var q = PlacesUtils.history.getNewQuery(),
+            o = PlacesUtils.history.getNewQueryOptions();
+        if (rootFolderId == PlacesUtils.tagsFolderId)
+            o.resultType = o.RESULTS_AS_TAG_QUERY;
+        else {
+            q.setFolders([rootFolderId], 1);
+            o.expandQueries = false;
+        }
+        var res = PlacesUtils.history.executeQuery(q, o);
+        var node = res.root;
+        for (let folder of folders) {
+            node = getFolderFromNode(node, folder);
+            if (!node)
+                break;
+        }
+        return node;
+
+        function getFolderFromNode (node, title) {
+            for (let child in Bookmarks.iterateFolderChildren(node)) {
+                if (child.title == title && child instanceof Ci.nsINavHistoryContainerResultNode) {
+                    node.containerOpen = false;
+                    return child;
+                }
+            }
+            return null;
+        }
+    },
+    iterateFolderChildren: function (node, onlyFolder, onlyWritable) {
+        if (!node.containerOpen)
+            node.containerOpen = true;
+
+        for (let i = 0, len = node.childCount; i < len; i++) {
+            let child = node.getChild(i);
+            if (PlacesUtils.nodeIsContainer(child))
+                child.QueryInterface(Ci.nsINavHistoryContainerResultNode);
+
+            if (PlacesUtils.nodeIsFolder(child)) {
+                if (onlyWritable && child.childrenReadOnly)
+                    continue;
+            }
+            else if (onlyFolder)
+                continue;
+
+            yield child;
+        }
+    },
 }, {
     commands: function () {
         commands.add(["ju[mps]"],
             "Show jumplist",
             function () {
-                // TODO: Make url clicking work again. Old code: <td><a href={val.URI.spec} highlight="URL jump-list">{val.URI.spec}</a></td>
                 let sh = history.session;
-                let jumps = [[idx == sh.index ? ">" : "", Math.abs(idx - sh.index), val.title, val.URI.spec] for ([idx, val] in Iterator(sh))];
-                let list = template.tabular([{ header: "Jump", style: "color: red", colspan: 2 }, { header: "", style: "text-align: right", highlight: "Number" },
+                let jumps = [[idx == sh.index ? ">" : "",
+                              Math.abs(idx - sh.index),
+                              val.title,
+                              let(url=val.URI.spec) xml`<a highlight="URL" href=${url}>${url}</a>`]
+                              for ([idx, val] in Iterator(sh))];
+                let list = template.tabular([{ header: "Jump", style: "color: red", colspan: 2 },
+                    { header: "", style: "text-align: right", highlight: "Number" },
                     { header: "Title", style: "width: 250px; max-width: 500px; overflow: hidden" },
                     { header: "URL", highlight: "URL jump-list" }],
                     jumps);
@@ -468,9 +525,9 @@ const Bookmarks = Module("bookmarks", {
             args.completeFilter = have.pop();
 
             let prefix = filter.substr(0, filter.length - args.completeFilter.length);
-            let tags = util.Array.uniq(util.Array.flatten([b.tags for ([k, b] in Iterator(bookmarks._cache.bookmarks))]));
+            let tags = util.Array.uniq(util.Array.flatten([b.tags for (b of bookmarks._cache.bookmarks)]));
 
-            return [[prefix + tag, tag] for ([i, tag] in Iterator(tags)) if (have.indexOf(tag) < 0)];
+            return [[prefix + tag, tag] for (tag of tags) if (have.indexOf(tag) < 0)];
         }
 
         function title(context, args) {
@@ -482,8 +539,8 @@ const Bookmarks = Module("bookmarks", {
         }
 
         function keyword(context, args) {
-            let keywords = util.Array.uniq(util.Array.flatten([b.keyword for ([k, b] in Iterator(bookmarks._cache.keywords))]));
-            return [[kw, kw] for ([i, kw] in Iterator(keywords)) ];
+            let keywords = util.Array.uniq(util.Array.flatten([b.keyword for (b of bookmarks._cache.keywords)]));
+            return [[kw, kw] for (kw of keywords)];
         }
 
         commands.add(["bma[rk]"],
@@ -493,8 +550,9 @@ const Bookmarks = Module("bookmarks", {
                 let title = args["-title"] || (args.length == 0 ? buffer.title : null);
                 let keyword = args["-keyword"] || null;
                 let tags =    args["-tags"] || [];
+                let folder =  args["-folder"] || "";
 
-                if (bookmarks.add(false, title, url, keyword, tags, args.bang)) {
+                if (bookmarks.add(false, title, url, keyword, tags, folder, args.bang)) {
                     let extra = (title == url) ? "" : " (" + title + ")";
                     liberator.echomsg("Added bookmark: " + url + extra);
                 }
@@ -512,7 +570,10 @@ const Bookmarks = Module("bookmarks", {
                 },
                 options: [[["-title", "-t"],    commands.OPTION_STRING, null, title],
                           [["-tags", "-T"],     commands.OPTION_LIST, null, tags],
-                          [["-keyword", "-k"],  commands.OPTION_STRING, function (arg) /\w/.test(arg)]]
+                          [["-keyword", "-k"],  commands.OPTION_STRING, function (arg) /\w/.test(arg)],
+                          [["-folder", "-f"],   commands.OPTION_STRING, null, function (context, args) {
+                              return completion.bookmarkFolder(context, args, PlacesUtils.bookmarksMenuFolderId, true, true);
+                          }]],
             });
 
         commands.add(["bmarks"],
@@ -604,6 +665,18 @@ const Bookmarks = Module("bookmarks", {
                     context.completions = [["", "Don't perform searches by default"]].concat(context.completions);
                 }
             });
+        var browserSearch = services.get("search");
+        browserSearch.init(function() {
+            var alias = browserSearch.defaultEngine.alias;
+            if (alias) {
+                let defsearch = options.get("defsearch");
+                // when already changed, it means the user had been changed in RC file
+                if (defsearch.value === defsearch.defaultValue)
+                    defsearch.value = alias
+
+                defsearch.defaultValue = alias;
+            }
+        });
     },
     completion: function () {
         completion.bookmark = function bookmark(context, tags, extra) {
@@ -627,7 +700,7 @@ const Bookmarks = Module("bookmarks", {
 
             context.title = ["Search Keywords"];
             context.completions = keywords.concat(engines);
-            context.keys = { text: 0, description: 1, icon: 2 };
+            context.keys = { text: 0, description: 1, icon: function(item) item[2] || DEFAULT_FAVICON };
 
             if (!space || noSuggest)
                 return;
@@ -672,9 +745,8 @@ const Bookmarks = Module("bookmarks", {
 
             let engineList = (engineAliases || options["suggestengines"] || "google").split(",");
 
-            let completions = [];
             engineList.forEach(function (name) {
-                let engine = services.get("browserSearch").getEngineByAlias(name);
+                let engine = services.get("search").getEngineByAlias(name);
                 if (!engine)
                     return;
                 let [, word] = /^\s*(\S+)/.exec(context.filter) || [];
@@ -685,6 +757,7 @@ const Bookmarks = Module("bookmarks", {
                 ctxt.title = [engine.description + " Suggestions"];
                 ctxt.compare = CompletionContext.Sort.unsorted;
                 ctxt.incomplete = true;
+                ctxt.match = function (str) str.toLowerCase().indexOf(this.filter.toLowerCase()) === 0;
                 bookmarks.getSuggestions(name, ctxt.filter, function (compl) {
                     ctxt.incomplete = false;
                     ctxt.completions = compl;
@@ -692,6 +765,40 @@ const Bookmarks = Module("bookmarks", {
             });
         };
 
+        completion.bookmarkFolder = function bookmarkFolder (context, args, rootFolderId, onlyFolder, onlyWritable) {
+            let filter = context.filter,
+                folders = filter.split("/"),
+                item = folders.pop();
+
+            if (!rootFolderId)
+            rootFolderId = PlacesUtils.bookmarksMenuFolderId;
+
+            let folderPath = folders.length ? folders.join("/") + "/" : "";
+            if (folders[0] === "TOOLBAR") {
+                rootFolderId = PlacesUtils.toolbarFolderId;
+                folders.shift();
+            }
+            let folder = Bookmarks.getBookmarkFolder(folders, rootFolderId);
+
+            context.title = ["Bookmarks", folders.join("/") + "/"];
+            context._match = function (filter, str) str.toLowerCase().startsWith(filter.toLowerCase());
+            let results = [];
+
+            if (folders.length === 0) {
+                results.push(["TOOLBAR", "Bookmarks Toolbar"]);
+            }
+            if (folder) {
+                for (let child in Bookmarks.iterateFolderChildren(folder, onlyFolder, onlyWritable)) {
+                    if (PlacesUtils.nodeIsSeparator(child))
+                        continue;
+
+                    results.push([folderPath + PlacesUIUtils.getBestTitle(child), child.uri]);
+                }
+                folder.containerOpen = false;
+            }
+            return context.completions = results;
+        };
+
         completion.addUrlCompleter("S", "Suggest engines", completion.searchEngineSuggest);
         completion.addUrlCompleter("b", "Bookmarks", completion.bookmark);
         completion.addUrlCompleter("s", "Search engines and keyword URLs", completion.search);
