@@ -70,7 +70,7 @@ const Buffer = Module("buffer", {
                     nFeed++;
                     let type = feedTypes[feed.type] || "RSS";
                     if (verbose)
-                        yield [feed.title, template.highlightURL(feed.href, true) + <span class="extra-info">&#xa0;({type})</span>];
+                        yield [feed.title, xml`${template.highlightURL(feed.href, true)}<span class="extra-info">&#xa0;(${type})</span>`];
                 }
             }
 
@@ -85,7 +85,7 @@ const Buffer = Module("buffer", {
             const ACCESS_READ = Ci.nsICache.ACCESS_READ;
             let cacheKey = doc.location.toString().replace(/#.*$/, "");
 
-            for (let proto in util.Array.itervalues(["HTTP", "FTP"])) {
+            for (let proto of ["HTTP", "FTP"]) {
                 try {
                     var cacheEntryDescriptor = services.get("cache").createSession(proto, 0, true)
                                                        .openCacheEntry(cacheKey, ACCESS_READ, false);
@@ -161,7 +161,7 @@ const Buffer = Module("buffer", {
 
     // called when the active document is scrolled
     _updateBufferPosition: function _updateBufferPosition() {
-        statusline.updateBufferPosition();
+        statusline.updateField("position");
         // modes.show();
     },
 
@@ -244,6 +244,7 @@ const Buffer = Module("buffer", {
         // for notifying the user about secure web pages
         onSecurityChange: function onSecurityChange(webProgress, request, state) {
             onSecurityChange.superapply(this, arguments);
+            statusline.updateField("ssl", state);
         },
         onStatusChange: function onStatusChange(webProgress, request, status, message) {
             onStatusChange.superapply(this, arguments);
@@ -254,14 +255,26 @@ const Buffer = Module("buffer", {
         // happens when the users switches tabs
         onLocationChange: function onLocationChange() {
             onLocationChange.superapply(this, arguments);
-            statusline.updateUrl();
-            statusline.updateBookmark();
-            statusline.updateHistory();
+            statusline.updateField("location");
+            statusline.updateField("bookmark");
+            statusline.updateField("history");
+
+            // This is a bit scary but we trigger ignore mode when the new URL is in the list
+            // of pages with ignored keys and then exit it on a new page but ONLY, if:
+            // a) The new page hasn't ignore as well and
+            // b) We initiated ignore mode, and not the user manually with <Insert>
+            if (modules.ignoreKeys != undefined) {
+                let ignoredKeyExceptions = ignoreKeys.hasIgnoredKeys(buffer.URL);
+                if (ignoredKeyExceptions !== null)
+                    modes.passAllKeysExceptSome(ignoredKeyExceptions);
+                else if (modes._passKeysExceptions !== null)
+                    modes.passAllKeys = false;
+            }
 
             autocommands.trigger("LocationChange", { url: buffer.URL });
 
             // if this is not delayed we get the position of the old buffer
-            setTimeout(function () { statusline.updateBufferPosition(); }, 250);
+            setTimeout(function () { statusline.updateField("position"); }, 250);
         },
         // called at the very end of a page load
         asyncUpdateUI: function asyncUpdateUI() {
@@ -272,8 +285,8 @@ const Buffer = Module("buffer", {
             let ssli = options["showstatuslinks"];
             if (link && ssli) {
                 if (ssli == 1) {
-                    statusline.updateUrl("Link: " + link);
-                    statusline.updateBookmark(link);
+                    statusline.updateField("location", "Link: " + link);
+                    statusline.updateField("bookmark", link);
                 }
                 else if (ssli == 2)
                     liberator.echo("Link: " + link, commandline.DISALLOW_MULTILINE);
@@ -281,8 +294,8 @@ const Buffer = Module("buffer", {
 
             if (link == "") {
                 if (ssli == 1) {
-                    statusline.updateUrl();
-                    statusline.updateBookmark();
+                    statusline.updateField("location");
+                    statusline.updateField("bookmark");
                 }
                 else if (ssli == 2)
                     modes.show();
@@ -373,6 +386,17 @@ const Buffer = Module("buffer", {
         return loc.href.substr(0, loc.href.length - loc.hash.length);
     },
 
+    /**
+     * @property {String} The current document's character set
+     */
+    get charset() {
+        try {
+            return config.browser.docShell.charset;
+        } catch (e) {
+            return "UTF-8";
+        }
+    },
+
     /**
      * @property {number} The buffer's height in pixels.
      */
@@ -540,7 +564,7 @@ const Buffer = Module("buffer", {
             }
 
             let res = util.evaluateXPath(options.get("hinttags").value, frame.document);
-            for (let [, regex] in Iterator(regexes)) {
+            for (let regex of regexes) {
                 for (let i in util.range(res.snapshotLength, 0, -1)) {
                     let elem = res.snapshotItem(i);
                     if (regex.test(elem.textContent) || regex.test(elem.title) ||
@@ -663,7 +687,7 @@ const Buffer = Module("buffer", {
             window.urlSecurityCheck(url, doc.nodePrincipal);
             // we always want to save that link relative to the current working directory
             options.setPref("browser.download.lastDir", io.getCurrentDirectory().path);
-            window.saveURL(url, text, null, true, skipPrompt, makeURI(url, doc.characterSet));
+            window.saveURL(url, text, null, true, skipPrompt, makeURI(url, doc.characterSet), doc);
         }
         catch (e) {
             liberator.echoerr(e);
@@ -846,7 +870,7 @@ const Buffer = Module("buffer", {
 
         // add the frame indicator
         let doc = frames[next].document;
-        let indicator = util.xmlToDom(<div highlight="FrameIndicator"/>, doc);
+        let indicator = util.xmlToDom(xml`<div highlight="FrameIndicator"/>`, doc);
         doc.body.appendChild(indicator);
 
         setTimeout(function () { doc.body.removeChild(indicator); }, 500);
@@ -864,7 +888,7 @@ const Buffer = Module("buffer", {
      * @param {Node} elem The element to query.
      */
     showElementInfo: function (elem) {
-        liberator.echo(<>Element:<br/>{util.objectToString(elem, true)}</>, commandline.FORCE_MULTILINE);
+        liberator.echo(xml`Element:<br/>${util.objectToString(elem, true)}`, commandline.FORCE_MULTILINE);
     },
 
     /**
@@ -881,23 +905,23 @@ const Buffer = Module("buffer", {
             let file = content.document.location.pathname.split("/").pop() || "[No Name]";
             let title = content.document.title || "[No Title]";
 
-            let info = template.map("gf",
-                function (opt) template.map(buffer.pageInfo[opt][0](), util.identity, ", "),
+            let info = template.map2(xml, "gf",
+                function (opt) template.map2(xml, buffer.pageInfo[opt][0](), util.identity, ", "),
                 ", ");
 
             if (bookmarks.isBookmarked(this.URL))
-                info += ", bookmarked";
+                xml["+="](info, ", bookmarked");
 
-            let pageInfoText = <>{file.quote()} [{info}] {title}</>;
+            let pageInfoText = xml`${file.quote()} [${info}] ${title}`;
             liberator.echo(pageInfoText, commandline.FORCE_SINGLELINE);
             return;
         }
 
         let option = sections || options["pageinfo"];
-        let list = template.map(option, function (option) {
+        let list = template.map2(xml, option, function (option) {
             let opt = buffer.pageInfo[option];
-            return opt ? template.table(opt[1], opt[0](true)) : undefined;
-        }, <br/>);
+            return opt ? template.table2(xml, opt[1], opt[0](true)) : undefined;
+        }, xml`<br/>`);
         liberator.echo(template.genericOutput("Page Information", list), commandline.FORCE_MULTILINE);
     },
 
@@ -940,7 +964,7 @@ const Buffer = Module("buffer", {
             editor.editFileExternally(url);
         else {
             const PREFIX = "view-source:";
-            if (url.indexOf(PREFIX) == 0)
+            if (url.startsWith(PREFIX))
                 url = url.substr(PREFIX.length);
             else
                 url = PREFIX + url;
@@ -976,41 +1000,34 @@ const Buffer = Module("buffer", {
         if (win === document.commandDispatcher.focusedWindow) return;
 
         // XXX: if win has frame and win is not content's focus window, win.focus() cannot focus.
-        if (win.frames.length && win !== this.focusedWindow) {
-            let html = win.document.documentElement;
-            let selection = win.getSelection();
-
-            let ranges = let(it = (function () {for (let i in util.range(0, selection.rangeCount)) yield selection.getRangeAt(i);})()) [r for(r in it)];
-
-            html.focus();
-            html.blur();
-
-            selection.removeAllRanges();
-            for (let [,r] in Iterator(ranges))
-                selection.addRange(r);
-            events.onFocusChange({});
-        } else
-            win.focus();
+        var e = win.document.activeElement;
+        if (e && e.contentWindow) {
+            services.get("focus").clearFocus(win);
+        }
+        win.focus();
     },
 
     getFocusedWindow: function (win) {
         win = win || config.browser.contentWindow;
         let elem = win.document.activeElement;
-        let doc;
-        while (doc = elem.contentDocument) {
-            elem = doc.activeElement;
-        }
-        return elem.ownerDocument.defaultView;
+        if (elem) {
+            let doc;
+            while (doc = elem.contentDocument) {
+                elem = doc.activeElement;
+            }
+            return elem.ownerDocument.defaultView;
+        } else
+            return win;
     },
 
-    setZoom: function setZoom(value, fullZoom) {
+    setZoom: function setZoom(value, fullZoom, browser = config.tabbrowser.mCurrentBrowser) {
         liberator.assert(value >= Buffer.ZOOM_MIN || value <= Buffer.ZOOM_MAX,
             "Zoom value out of range (" + Buffer.ZOOM_MIN + " - " + Buffer.ZOOM_MAX + "%)");
 
         ZoomManager.useFullZoom = fullZoom;
         ZoomManager.zoom = value / 100;
         if ("FullZoom" in window)
-            FullZoom._applySettingToPref();
+            FullZoom._applyZoomToPref(browser);
         liberator.echomsg((fullZoom ? "Full" : "Text") + " zoom: " + value + "%");
     },
 
@@ -1048,23 +1065,18 @@ const Buffer = Module("buffer", {
     },
 
     findScrollable: function findScrollable(dir, horizontal) {
-        let pos = "scrollTop", size = "clientHeight", max = "scrollHeight", layoutSize = "offsetHeight",
-            overflow = "overflowX", border1 = "borderTopWidth", border2 = "borderBottomWidth";
+        let pos = "scrollTop", maxPos = "scrollTopMax", clientSize = "clientHeight";
         if (horizontal)
-            pos = "scrollLeft", size = "clientWidth", max = "scrollWidth", layoutSize = "offsetWidth",
-            overflow = "overflowX", border1 = "borderLeftWidth", border2 = "borderRightWidth";
+            pos = "scrollLeft", maxPos = "scrollLeftMax", clientSize = "clientWidth";
 
         function find(elem) {
+            if (elem && !(elem instanceof Element))
+                elem = elem.parentNode;
+
             for (; elem && elem.parentNode instanceof Element; elem = elem.parentNode) {
-                let style = util.computedStyle(elem);
-                let borderSize = parseInt(style[border1]) + parseInt(style[border2]);
-                let realSize = elem[size];
-                // Stupid Gecko eccentricities. May fail for quirks mode documents.
-                if (elem[size] + borderSize == elem[max] || elem[size] == 0) // Stupid, fallible heuristic.
+                if (elem[clientSize] == 0)
                     continue;
-                if (style[overflow] == "hidden")
-                    realSize += borderSize;
-                if (dir < 0 && elem[pos] > 0 || dir > 0 && elem[pos] + realSize < elem[max])
+                if (dir < 0 && elem[pos] > 0 || dir > 0 && elem[pos] < elem[maxPos])
                     break;
             }
             return elem;
@@ -1152,7 +1164,7 @@ const Buffer = Module("buffer", {
         commands.add(["frameo[nly]"],
             "Show only the current frame's page",
             function (args) {
-                liberator.open(tabs.localStore.focusedFrame.document.documentURI);
+                liberator.open(buffer.localStore.focusedFrame.document.documentURI);
             },
             { argCount: "0" });
 
@@ -1262,7 +1274,7 @@ const Buffer = Module("buffer", {
 
                 window.internalSave(doc.location.href, doc, null, contentDisposition,
                     doc.contentType, false, null, chosenData, doc.referrer ?
-                    window.makeURI(doc.referrer) : null, true);
+                    window.makeURI(doc.referrer) : null, doc, true);
             },
             {
                 argCount: "?",
@@ -1343,9 +1355,7 @@ const Buffer = Module("buffer", {
         }
         function getURLFromTab (tab) {
             if ("linkedBrowser" in tab)
-                return ("__SS_restoreState" in tab.linkedBrowser && "__SS_data" in tab.linkedBrowser) ?
-                    tab.linkedBrowser.__SS_data.entries.slice(-1)[0].url :
-                    tab.linkedBrowser.contentDocument.location.href;
+                return tab.linkedBrowser.contentDocument.location.href;
             else {
                 let i = config.tabbrowser.mTabContainer.getIndexOfItem(tab);
                 let info = config.tabbrowser.tabInfo[i];
@@ -1355,8 +1365,14 @@ const Buffer = Module("buffer", {
             }
         }
 
-        function createItem (prefix, label, url, indicator, icon)
-            ({ text: [prefix + label, prefix + url], url: template.highlightURL(url), indicator: indicator, icon: icon || DEFAULT_FAVICON })
+        function createItem (prefix, label, url, indicator, icon) {
+            return {
+                text: [prefix + label, prefix + url],
+                url: template.highlightURL(url, true, "buffer-list"),
+                indicator: indicator,
+                icon: icon || DEFAULT_FAVICON
+            };
+        }
 
         function generateTabs (tabs) {
             for (let i = 0, tab; tab = tabs[i]; i++) {
@@ -1395,10 +1411,10 @@ const Buffer = Module("buffer", {
             context.compare = CompletionContext.Sort.number;
             let process = context.process[0];
             context.process = [function (item, text)
-                    <>
-                        <span highlight="Indicator" style="display: inline-block; width: 2ex; padding-right: 0.5ex; text-align: right">{item.item.indicator}</span>
-                        { process.call(this, item, text) }
-                    </>];
+                    xml`
+                        <span highlight="Indicator" style="display: inline-block; width: 2ex; padding-right: 0.5ex; text-align: right">${item.item.indicator}</span>
+                        ${ process.call(this, item, text) }
+                    `];
 
             let tabs;
             if (!flags) {
@@ -1418,7 +1434,7 @@ const Buffer = Module("buffer", {
             if (flags & this.buffer.GROUPS) {
                 let activeGroup = groups.getActiveGroupItem();
                 let activeGroupId = activeGroup === null ? null : activeGroup.id;
-                for (let [i, group] in Iterator(groups.groupItems)) {
+                for (let group of groups.groupItems) {
                     if (group.id != activeGroupId) {
                         let groupName = group.getTitle();
                         context.fork("GROUP_" + group.id, 0, this, function (context) {
@@ -1456,7 +1472,7 @@ const Buffer = Module("buffer", {
               .XULBrowserWindow = this.progressListener;
 
         try {
-            config.browser.addProgressListener(this.progressListener, Ci.nsIWebProgress.NOTIFY_ALL);
+            config.browser.addProgressListener(this.progressListener);
         }
         catch (e) {} // Why? --djk
 
@@ -1486,10 +1502,12 @@ const Buffer = Module("buffer", {
                 if (Editor.windowIsEditable()) {
                     if (options["insertmode"])
                         modes.set(modes.INSERT);
-                    else if (win.getSelection().toString() != "")
+                    else if (Buffer.focusedWindow.getSelection().toString() != "")
                         modes.set(modes.VISUAL, modes.TEXTAREA);
-                    else
+                    else {
+                        options.setPref("accessibility.browsewithcaret", true);
                         modes.main = modes.TEXTAREA;
+                    }
                 } else {
                     // setting this option notifies an observer which takes care of the
                     // mode setting
