@@ -29,8 +29,8 @@ const CommandLine = Module("commandline", {
                 CLEAR: "browser:purge-session-history",
                 QUIT:  "quit-application",
                 init: function () {
-                    services.get("observer").addObserver(this, this.CLEAR, false);
-                    services.get("observer").addObserver(this, this.QUIT, false);
+                    services.get("obs").addObserver(this, this.CLEAR, false);
+                    services.get("obs").addObserver(this, this.QUIT, false);
                 },
                 observe: function (subject, topic, data) {
                     switch (topic) {
@@ -40,8 +40,8 @@ const CommandLine = Module("commandline", {
                         });
                         break;
                     case this.QUIT:
-                        services.get("observer").removeObserver(this, this.CLEAR);
-                        services.get("observer").removeObserver(this, this.QUIT);
+                        services.get("obs").removeObserver(this, this.CLEAR);
+                        services.get("obs").removeObserver(this, this.QUIT);
                         break;
                     }
                 }
@@ -161,6 +161,7 @@ const CommandLine = Module("commandline", {
         this._multilineCallback = null;
 
         this._input = {};
+        this._keepOpenForInput = false;
 
         this._commandlineDisplayTimeoutID = null;
 
@@ -206,6 +207,7 @@ const CommandLine = Module("commandline", {
         }
 
         function closePrompt(value) {
+            self._keepOpenForInput = false;
             let callback = self._input.submit;
             self._input = {};
             if (callback)
@@ -223,6 +225,14 @@ const CommandLine = Module("commandline", {
         document.getElementById('liberator-statusline').setAttributeNS(NS.uri, "highlight", group);
     },
 
+    /**
+     * reset messasaBox
+     */
+    _resetStatusline: function () {
+        this._messageBox.value = "";
+        this._setHighlightGroup("");
+    },
+
     /**
      * Determines whether the command line should be visible.
      *
@@ -301,14 +311,8 @@ const CommandLine = Module("commandline", {
 
         liberator.triggerObserver("echoMultiline", str, highlightGroup);
 
-        // If it's already XML, assume it knows what it's doing.
-        // Otherwise, white space is significant.
-        // The problem elsewhere is that E4X tends to insert new lines
-        // after interpolated data.
-        XML.ignoreWhitespace = typeof str != "xml";
-        this._lastMowOutput = <div class="ex-command-output" style="white-space: nowrap" highlight={highlightGroup}>{template.maybeXML(str)}</div>;
+        this._lastMowOutput = xml`<div class="ex-command-output" style="white-space: nowrap" highlight=${highlightGroup}>${template.maybeXML(str)}</div>`;
         let output = util.xmlToDom(this._lastMowOutput, doc);
-        XML.ignoreWhitespace = true;
 
         // FIXME: need to make sure an open MOW is closed when commands
         //        that don't generate output are executed
@@ -329,7 +333,6 @@ const CommandLine = Module("commandline", {
 
         win.focus();
 
-        this._startHints = false;
         modes.set(modes.COMMAND_LINE, modes.OUTPUT_MULTILINE);
     },
 
@@ -474,6 +477,8 @@ const CommandLine = Module("commandline", {
         // open the completion list automatically if wanted
         if (cmd.length)
             commandline.triggerCallback("change", this._currentExtendedMode, cmd);
+
+        this._resetStatusline();
     },
 
     /**
@@ -482,6 +487,7 @@ const CommandLine = Module("commandline", {
      * called directly after a successful command, otherwise it will.
      */
     close: function close() {
+        this._keepOpenForInput = false;
         let mode = this._currentExtendedMode;
         this._currentExtendedMode = null;
         commandline.triggerCallback("cancel", mode); // FIXME
@@ -497,6 +503,14 @@ const CommandLine = Module("commandline", {
         this._completions = null;
 
         // liberator.log('closing with : ' + modes.main + "/" + modes.extended);
+
+        // hint prompt is available
+        if (this._startHints && modes.extended & modes.PROMPT) {
+            this._startHints = false;
+            return;
+        }
+        this._startHints = false;
+
         // don't have input and output widget open at the same time
         if (modes.extended & modes.INPUT_MULTILINE)
             this._outputContainer.collapsed = true;
@@ -507,6 +521,8 @@ const CommandLine = Module("commandline", {
             liberator.focusContent();
             this._multilineInputWidget.collapsed = true;
             this._outputContainer.collapsed = true;
+            if (this._messageBox.getAttributeNS(NS.uri, "highlight") === this.HL_MOREMSG)
+                this._messageBox.value = "";
             this.hide();
             //modes.pop(true);
             modes.reset();
@@ -593,14 +609,14 @@ const CommandLine = Module("commandline", {
                 action = this._echoMultiline;
             }
 
-            if ((flags & this.FORCE_MULTILINE) || (/\n/.test(str) || typeof str == "xml") && !(flags & this.FORCE_SINGLELINE))
+            if ((flags & this.FORCE_MULTILINE) || (/\n/.test(str) || typeof str == "xml" || str instanceof TemplateSupportsXML ) && !(flags & this.FORCE_SINGLELINE))
                 action = this._echoMultiline;
 
             if (single)
                 this._lastEcho = null;
             else {
                 if (this._messageBox.value == this._lastEcho)
-                    this._echoMultiline(<span highlight="Message">{this._lastEcho}</span>,
+                    this._echoMultiline(xml`<span highlight="Message">${this._lastEcho}</span>`,
                         this._messageBox.getAttributeNS(NS.uri, "highlight"));
                 this._lastEcho = (action == this._echoLine) && str;
             }
@@ -635,6 +651,7 @@ const CommandLine = Module("commandline", {
             complete: extra.completer,
             cancel: extra.onCancel
         };
+        this._keepOpenForInput = true;
 
         modes.set(modes.COMMAND_LINE, modes.PROMPT);
         this._currentExtendedMode = modes.PROMPT;
@@ -645,6 +662,7 @@ const CommandLine = Module("commandline", {
         this.show();
         this._commandWidget.focus();
 
+        this._history = CommandLine.History(this._commandWidget.inputField, (modes.extended == modes.EX) ? "command" : "search");
         this._completions = CommandLine.Completions(this._commandWidget.inputField);
     },
 
@@ -691,9 +709,8 @@ const CommandLine = Module("commandline", {
             }, 0);
         }
         else if (event.type == "focus") {
-            if (!this._commandShown() && event.target == this._commandWidget.inputField) {
-                event.target.blur();
-                liberator.beep();
+            if (!this._commandShown() && event.originalTarget == this._commandWidget.inputField) {
+                event.originalTarget.blur();
             }
         }
         else if (event.type == "input") {
@@ -714,7 +731,8 @@ const CommandLine = Module("commandline", {
                 //let currentExtendedMode = this._currentExtendedMode;
                 //this._currentExtendedMode = null; // Don't let modes.pop trigger "cancel"
                 commandline.triggerCallback("submit", this._currentExtendedMode, command);
-                this.close();
+                if (!this._keepOpenForInput)
+                    this.close();
             }
             // user pressed <Up> or <Down> arrow to cycle this._history completion
             else if (/^<(Up|Down|S-Up|S-Down|PageUp|PageDown)>$/.test(key)) {
@@ -813,7 +831,8 @@ const CommandLine = Module("commandline", {
                 case "<LeftMouse>":
                     if (event.originalTarget.getAttributeNS(NS.uri, "highlight") == "URL buffer-list") {
                         event.preventDefault();
-                        tabs.select(parseInt(event.originalTarget.parentNode.parentNode.firstChild.textContent, 10) - 1, false, true);
+                        let textContent = event.originalTarget.parentNode.parentNode.firstElementChild.textContent.trim();
+                        tabs.select(parseInt(textContent.replace(/^[^\d]*/, ""), 10) - 1, false, true);
                     }
                     else
                         openLink(liberator.CURRENT_TAB);
@@ -836,16 +855,7 @@ const CommandLine = Module("commandline", {
             return;
         }
 
-        if (this._startHints) {
-            statusline.updateInputBuffer("");
-            this._startHints = false;
-            hints.show(key, undefined, win);
-            return;
-        }
-
-        function isScrollable() { if (win.scrollMaxY == 0) liberator.beep(); return !win.scrollMaxY == 0; }
-        function atEnd() win.scrollY / win.scrollMaxY >= 1;
-
+        let isScrollable = win.scrollMaxY !== 0;
         let showHelp = false;
         switch (key) {
             // close the window
@@ -861,9 +871,9 @@ const CommandLine = Module("commandline", {
 
             // extended hint modes
             case ";":
-                statusline.updateInputBuffer(";");
                 this._startHints = true;
-                break;
+                hints.startExtendedHint("", win)
+                return;
 
             // down a line
             case "j":
@@ -871,28 +881,36 @@ const CommandLine = Module("commandline", {
             case "<C-j>":
             case "<C-m>":
             case "<Return>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollByLines(1);
+                else
+                    showHelp = true;
                 break;
 
             // up a line
             case "k":
             case "<Up>":
             case "<BS>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollByLines(-1);
+                else
+                    showHelp = true;
                 break;
 
             // half page down
             case "d":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollBy(0, win.innerHeight / 2);
+                else
+                    showHelp = true;
                 break;
 
             // half page up
             case "u":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollBy(0, -(win.innerHeight / 2));
+                else
+                    showHelp = true;
                 break;
 
             // page down
@@ -900,32 +918,47 @@ const CommandLine = Module("commandline", {
             case "<C-f>":
             case "<Space>":
             case "<PageDown>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollByPages(1);
+                else
+                    showHelp = true;
                 break;
 
             // page up
             case "b":
             case "<C-f>":
             case "<PageUp>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollByPages(-1);
+                else
+                    showHelp = true;
                 break;
 
             // top of page
             case "g":
             case "<Home>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollTo(0, 0);
+                else
+                    showHelp = true;
                 break;
 
             // bottom of page
             case "G":
             case "<End>":
-                if (isScrollable())
+                if (isScrollable)
                     win.scrollTo(0, win.scrollMaxY);
+                else
+                    showHelp = true;
+                break;
+            case "Y":
+                let (sel = win.getSelection().toString()) {
+                    if (sel)
+                        util.copyToClipboard(sel, false);
+                    else
+                        showHelp = true;
+                }
                 break;
-
             // unmapped key -> show Help
             default:
                 showHelp = true;
@@ -933,7 +966,13 @@ const CommandLine = Module("commandline", {
 
         if (showHelp) {
             this.hide(); // hide the command line
-            this._echoLine("SPACE/d/j: screen/page/line down | b/u/k: screen/page/line up | HOME/g: top | END/G: bottom | ;f: follow hint | ESC/q: quit", this.HL_MOREMSG, true);
+            let helpMsg = "Y: yank | ;o: follow hint | ESC/q: quit";
+            if (isScrollable)
+                helpMsg = "SPACE/d/j: screen/page/line down | b/u/k: screen/page/line up | " +
+                          "HOME/g: top | END/G: bottom | " +
+                          helpMsg;
+
+            this._echoLine(helpMsg, this.HL_MOREMSG, true);
         } else {
             this.show();
         }
@@ -1195,7 +1234,7 @@ const CommandLine = Module("commandline", {
             }
 
             // Don't show 1-character substrings unless we've just hit backspace
-            if (substring.length < 2 && (!this._lastSubstring || this._lastSubstring.indexOf(substring) != 0))
+            if (substring.length < 2 && (!this._lastSubstring || !this._lastSubstring.startsWith(substring)))
                 return;
             this._lastSubstring = substring;
 
@@ -1205,7 +1244,7 @@ const CommandLine = Module("commandline", {
             substring = substring.substr(value.length);
             this.removeSubstring = substring;
 
-            let node = util.xmlToDom(<span highlight="Preview">{substring}</span>,
+            let node = util.xmlToDom(xml`<span highlight="Preview">${substring}</span>`,
                 document);
             let start = this.caret;
             this.editor.insertNode(node, this.editor.rootElement, 1);
@@ -1299,7 +1338,7 @@ const CommandLine = Module("commandline", {
                 let n = 0;
                 try {
                     this.waiting = true;
-                    for (let [, context] in Iterator(list)) {
+                    for (let context of list) {
                         function done() !(idx >= n + context.items.length || idx == -2 && !context.items.length);
                         while (context.incomplete && !done())
                             liberator.threadYield(false, true);
@@ -1392,13 +1431,14 @@ const CommandLine = Module("commandline", {
             return null;
         }
 
-        if (typeof arg === "object")
+        if (arg instanceof TemplateSupportsXML);
+        else if (typeof arg === "object")
             arg = util.objectToString(arg, useColor);
         else
             arg = String(arg);
 
         if (typeof arg == "string" && /\n/.test(arg))
-            arg = <span highlight="CmdOutput">{arg}</span>;
+            arg = xml`<span highlight="CmdOutput">${arg}</span>`;
 
         return arg;
     }
@@ -1424,7 +1464,9 @@ const CommandLine = Module("commandline", {
             commands.add([command.name],
                 command.description,
                 function (args) {
-                    let str = CommandLine.echoArgumentToString(args.string, true);
+                    var str = args.literalArg;
+
+                    let str = CommandLine.echoArgumentToString(str, true);
                     if (str != null)
                         command.action(str);
                 }, {
@@ -1445,9 +1487,8 @@ const CommandLine = Module("commandline", {
                     commandline.echo(message.str, message.highlight, commandline.FORCE_SINGLELINE);
                 }*/
                 else { //if (commandline._messageHistory.length > 1) {
-                    XML.ignoreWhitespace = false;
-                    let list = template.map(commandline._messageHistory.messages, function (message)
-                        <div highlight={message.highlight + " Message"}>{message.str}</div>);
+                    let list = template.map2(xml, commandline._messageHistory.messages, function (message)
+                        xml`<div highlight=${message.highlight + " Message"}>${message.str}</div>`);
                     liberator.echo(list, commandline.FORCE_MULTILINE);
                 }
             },
@@ -1555,7 +1596,7 @@ const CommandLine = Module("commandline", {
              "stringlist", "google",
              {
                  completer: function completer(value) {
-                     let engines = services.get("browserSearch").getEngines({})
+                     let engines = services.get("search").getEngines({})
                                            .filter(function (engine) engine.supportsResponseType("application/x-suggestions+json"));
 
                      return engines.map(function (engine) [engine.alias, engine.description]);
@@ -1669,18 +1710,18 @@ const ItemList = Class("ItemList", {
 
     _init: function () {
         this._div = this._dom(
-            <div class="ex-command-output" highlight="Normal" style="white-space: nowrap">
+            xml`<div class="ex-command-output" highlight="Normal" style="white-space: nowrap">
                 <div highlight="Completions" key="noCompletions"><span highlight="Title">No Completions</span></div>
                 <div key="completions"/>
                 <div highlight="Completions">
-                {
-                    template.map(util.range(0, options["maxitems"] * 2), function (i)
-                    <span highlight="CompItem">
+                ${
+                    template.map2(xml, util.range(0, options["maxitems"] * 2), function (i)
+                    xml`<span highlight="CompItem">
                         <li highlight="NonText">~</li>
-                    </span>)
+                    </span>`)
                 }
                 </div>
-            </div>, this._divNodes);
+            </div>`, this._divNodes);
         this._doc.body.replaceChild(this._div, this._doc.body.firstChild);
         //div.scrollIntoView(true);
 
@@ -1689,14 +1730,14 @@ const ItemList = Class("ItemList", {
             if (!context.items.length && !context.message && !context.incomplete)
                 return;
             context.cache.nodes = [];
-            this._dom(<div key="root" highlight="CompGroup">
+            this._dom(xml`<div key="root" highlight="CompGroup">
                     <div highlight="Completions">
-                        { context.createRow(context.title || [], "CompTitle") }
+                        ${ context.createRow(context.title || [], "CompTitle") }
                     </div>
                     <div key="message" highlight="CompMsg"/>
                     <div key="items" highlight="Completions"/>
-                    <div key="waiting" highlight="CompMsg">{ItemList.WAITING_MESSAGE}</div>
-                </div>, context.cache.nodes);
+                    <div key="waiting" highlight="CompMsg">${ItemList.WAITING_MESSAGE}</div>
+                </div>`, context.cache.nodes);
             this._divNodes.completions.appendChild(context.cache.nodes.root);
         }, this);
 
@@ -1710,7 +1751,6 @@ const ItemList = Class("ItemList", {
      * @param {number} offset Start at this index and show options["maxitems"].
      */
     _fill: function (offset) {
-        XML.ignoreWhiteSpace = false;
         let diff = offset - this._startIndex;
         if (this._items == null || offset == null || diff == 0 || offset < 0)
             return false;
