@@ -406,7 +406,7 @@ const CompletionContext = Class("CompletionContext", {
         let filter = fixCase(this.filter);
         if (this.anchored) {
             var compare = function compare(text, s) text.substr(0, s.length) == s;
-            substrings = util.map(util.range(filter.length, text.length + 1),
+            var substrings = util.map(util.range(filter.length, text.length + 1),
                 function (end) text.substring(0, end));
         }
         else {
@@ -456,7 +456,7 @@ const CompletionContext = Class("CompletionContext", {
     },
 
     cancelAll: function () {
-        for (let [, context] in Iterator(this.contextList)) {
+        for (let context of this.contextList) {
             if (context.cancel)
                 context.cancel();
         }
@@ -593,7 +593,7 @@ const CompletionContext = Class("CompletionContext", {
         }
         //for (let key in (k for ([k, v] in Iterator(self.contexts)) if (v.offset > this.caret)))
         //    delete this.contexts[key];
-        for each (let context in this.contexts) {
+        for (let [, context] in Iterator(this.contexts)) {
             context.hasItems = false;
             if (context != context.top)
                 context.incomplete = false;
@@ -677,12 +677,12 @@ const Completion = Module("completion", {
         context.wait();
 
         for (let [key, context] in Iterator(context.contexts)) {
-            if (key.indexOf("/list") == 0) {
+            if (key.startsWith("/list")) {
                 let list = template.genericOutput("",
-                    <div highlight="Completions">
-                        { template.completionRow(context.title, "CompTitle") }
-                        { template.map(context.items, function (item) context.createRow(item), null, 100) }
-                    </div>);
+                    xml`<div highlight="Completions">
+                        ${ template.completionRow(context.title, "CompTitle") }
+                        ${ template.map2(xml, context.items, function (item) context.createRow(item), null, 100) }
+                    </div>`);
                 commandline.echo(list, commandline.HL_NORMAL, commandline.FORCE_MULTILINE);
             }
         }
