@@ -94,7 +94,7 @@ const Editor = Module("editor", {
 
     // count is optional, defaults to 1
     executeCommand: function (cmd, count) {
-        let controller = Editor.getController();
+        let controller = Editor.getController(cmd);
         if (!controller || !controller.supportsCommand(cmd) || !controller.isCommandEnabled(cmd)) {
             liberator.beep();
             return false;
@@ -294,7 +294,7 @@ const Editor = Module("editor", {
     },
 
     // TODO: clean up with 2 functions for textboxes and currentEditor?
-    editFieldExternally: function (forceEditing) {
+    editFieldExternally: function (forceEditing, field) {
         if (!options["editor"])
             return;
 
@@ -310,14 +310,18 @@ const Editor = Module("editor", {
             nsEditor instanceof Ci.nsIHTMLEditor;
         }
         else {
-            textBox = liberator.focus;
+	    if (field !== undefined) {
+		    textBox = field; // if I reply 'yes', textBox.value == 'yes', so get the real value
+	    } else {
+		    textBox = liberator.focus;
+	    }
         }
 
         if (!forceEditing && textBox && textBox.type == "password") {
             commandline.input("Editing a password field externally will reveal the password. Would you like to continue? (yes/[no]): ",
                 function (resp) {
                     if (resp && resp.match(/^y(es)?$/i))
-                        editor.editFieldExternally(true);
+                        editor.editFieldExternally(true, textBox);
                 });
                 return;
         }
@@ -373,7 +377,6 @@ const Editor = Module("editor", {
                         nsEditor.selection.deleteFromDocument();
                         if (isHTML) {
                             let doc = nsEditor.document;
-                            let htmlFragment = doc.implementation.createDocument(null, 'html', null);
                             let range = doc.createRange();
                             range.setStartAfter(doc.body);
                             doc.body.appendChild(range.createContextualFragment(val));
@@ -381,6 +384,13 @@ const Editor = Module("editor", {
                         else {
                             nsEditor.insertText(val);
                         }
+
+                        let lastChild = rootNode.lastChild;
+                        while (lastChild.localName === "br" &&
+                                (lastChild.hasAttribute("_moz_editor_bogus_node") || lastChild.hasAttribute("type"))) {
+                            nsEditor.rootElement.removeChild(lastChild)
+                            lastChild = rootNode.lastChild;
+                        }
                     }
                 }
                 let timer = services.create("timer");
@@ -468,20 +478,24 @@ const Editor = Module("editor", {
 }, {
     getEditor: function () {
         let e = liberator.focus;
-        if (!e) {
+        if (!e || e.isContentEditable) {
             e = document.commandDispatcher.focusedWindow;
             if (!Editor.windowIsEditable(e))
                 return null;
+        } else if (config.isComposeWindow
+            && document.compareDocumentPosition(e) & Node.DOCUMENT_POSITION_DISCONNECTED
+            && window.GetCurrentEditor) {
+            return window.GetCurrentEditor().document.defaultView;
         }
         return e;
     },
 
-    getController: function () {
+    getController: function (cmd) {
         let ed = Editor.getEditor();
         if (!ed || !ed.controllers)
             return null;
 
-        return ed.controllers.getControllerForCommand("cmd_beginLine");
+        return ed.controllers.getControllerForCommand(cmd || "cmd_beginLine");
     },
 
     windowIsEditable: function (win) {
@@ -495,8 +509,8 @@ const Editor = Module("editor", {
             .QueryInterface(Ci.nsIInterfaceRequestor)
             .getInterface(Ci.nsIEditingSession);
         return editingSession.windowIsEditable(win) &&
-               win.document.body &&
-               util.computedStyle(win.document.body).getPropertyValue("-moz-user-modify") == "read-write";
+               (win.document.designMode === "on" ||
+                    (services.get("focus").getFocusedElementForWindow(win, true, {})||{}).isContentEditable);
     }
 }, {
     mappings: function () {
@@ -621,8 +635,7 @@ const Editor = Module("editor", {
                 // broken in FF3, deletes the whole line:
                 // editor.executeCommand("cmd_deleteToBeginningOfLine", 1);
                 editor.executeCommand("cmd_selectBeginLine", 1);
-                if (Editor.getController().isCommandEnabled("cmd_delete"))
-                    editor.executeCommand("cmd_delete", 1);
+                editor.executeCommand("cmd_delete", 1);
             });
 
         mappings.add(myModes,
