@@ -54,7 +54,7 @@ const Events = Module("events", {
                 if (k in this._keyTable)
                     names = this._keyTable[k];
                 this._code_key[v] = names[0];
-                for (let [, name] in Iterator(names))
+                for (let name of names)
                     this._key_code[name.toLowerCase()] = v;
             }
 
@@ -79,7 +79,7 @@ const Events = Module("events", {
                 let dirs = io.getRuntimeDirectories("macros");
 
                 if (dirs.length > 0) {
-                    for (let [, dir] in Iterator(dirs)) {
+                    for (let dir of dirs) {
                          liberator.log("Sourcing macros directory: " + dir.path);
 
                         for (let file in dir.iterDirectory()) {
@@ -282,7 +282,7 @@ const Events = Module("events", {
         try {
             feed: {
                 liberator.threadYield(1, true);
-                for (let [, evt_obj] in Iterator(events.fromString(keys))) {
+                for (let evt_obj of events.fromString(keys)) {
                     let elem = liberator.focus || config.browser.contentWindow;
                     let evt = events.create(doc, "keypress", evt_obj);
 
@@ -320,7 +320,7 @@ const Events = Module("events", {
             if (this.duringFeed.length) {
                 let duringFeed = this.duringFeed;
                 this.duringFeed = [];
-                for (let [, evt] in Iterator(duringFeed))
+                for (let evt of duringFeed)
                     evt.target.dispatchEvent(evt);
             }
         }
@@ -364,8 +364,7 @@ const Events = Module("events", {
         };
         var t = TYPES[type];
         var evt = doc.createEvent(t + "Events");
-        evt["init" + t + "Event"].apply(evt,
-                [v for ([k, v] in Iterator(util.extend(DEFAULTS[t], opts)))]);
+        evt["init" + t + "Event"].apply(evt, [v for (v of values(update(DEFAULTS[t], opts)))]);
         return evt;
     },
 
@@ -654,8 +653,8 @@ const Events = Module("events", {
         let win  = window.document.commandDispatcher.focusedWindow;
         let elem = window.document.commandDispatcher.focusedElement;
 
-        if (win && win.top == content && liberator.has("tabs"))
-            tabs.localStore.focusedFrame = win;
+        if (win && win.top == content)
+            buffer.localStore.focusedFrame = win;
 
         try {
             if (elem && elem.readOnly)
@@ -741,7 +740,7 @@ const Events = Module("events", {
      *  The global escape key handler. This is called in ALL modes.
      */
     onEscape: function () {
-        if (modes.passNextKey || modes.passAllKeys)
+        if (modes.passNextKey) // We allow <Esc> in passAllKeys
             return;
 
         // always clear the commandline. Even if we are in
@@ -892,7 +891,16 @@ const Events = Module("events", {
             } else if (modes.passAllKeys) { // handle Escape-all-keys mode (Shift-Esc)
                 if (key == "<S-Esc>" || key == "<Insert>") // FIXME: Don't hardcode!
                     modes.passAllKeys = false;
-                stop = true;
+
+                // If we manage to get into command line mode while IGNOREKEYS, let the command line handle keys
+                if (liberator.mode == modes.COMMAND_LINE)
+                    stop = false;
+                // If we are in the middle of a mapping, we never send the next key to the host app
+                else if (this._input.buffer) 
+                    stop = false;
+                // Respect "unignored" keys
+                else if (modes._passKeysExceptions == null || modes._passKeysExceptions.indexOf(key) < 0)
+                    stop = true;
             }
 
             if (stop) {
@@ -1053,13 +1061,13 @@ const Events = Module("events", {
         finally {
             let motionMap = (this._input.pendingMotionMap && this._input.pendingMotionMap.names[0]) || "";
             if (!(modes.extended & modes.HINTS))
-                statusline.updateInputBuffer(motionMap + this._input.buffer);
+                statusline.updateField("input", motionMap + this._input.buffer);
         }
     },
 
     // this is need for sites like msn.com which focus the input field on keydown
     onKeyUpOrDown: function (event) {
-        if (modes.passNextKey || modes.passAllKeys || Events.isInputElemFocused())
+        if (modes.passNextKey || modes.passAllKeys || modes.isMenuShown || Events.isInputElemFocused())
             return;
         event.stopPropagation();
     },
@@ -1070,7 +1078,9 @@ const Events = Module("events", {
         modes.isMenuShown = true;
     },
 
-    onPopupHidden: function () {
+    onPopupHidden: function (event) {
+        if (event.originalTarget.localName == "tooltip" || event.originalTarget.id == "liberator-visualbell")
+            return;
         // gContextMenu is set to NULL, when a context menu is closed
         if ((window.gContextMenu == null || !window.gContextMenu.shouldDisplay) && !this._activeMenubar)
             modes.isMenuShown = false;
@@ -1141,7 +1151,7 @@ const Events = Module("events", {
         // add the ":" mapping in all but insert mode mappings
         mappings.add(modes.matchModes({ extended: false, input: false }),
             [":"], "Enter command line mode",
-            function () { commandline.open("", "", modes.EX); });
+            function (count) { commandline.open("", count ? String(count) : "", modes.EX); }, {count: true});
 
         // focus events
         mappings.add([modes.NORMAL, modes.PLAYER, modes.VISUAL, modes.CARET],
