@@ -21,7 +21,8 @@ const Hints = Module("hints", {
         this._extendedhintCount = null;  // for the count argument of Mode#action (extended hint only)
 
         this._pageHints = [];
-        this._validHints = []; // store the indices of the "hints" array with valid elements
+        this._validHints = [];           // store the indices of the "hints" array with valid elements
+        this._tabNavigation = {};        // for navigating between valid hints when TAB key is pressed
 
         this._activeTimeout = null;  // needed for hinttimeout > 0
         this._canUpdate = false;
@@ -81,13 +82,14 @@ const Hints = Module("hints", {
      * Reset hints, so that they can be cleanly used again.
      */
     _reset: function () {
-        statusline.updateInputBuffer("");
+        statusline.updateField("input", "");
         this._hintString = "";
         this._hintNumber = 0;
         this._usedTabKey = false;
         this._prevInput = "";
         this._pageHints = [];
         this._validHints = [];
+        this._tabNavigation = {};
         this._canUpdate = false;
         this._docs = [];
         hints.escNumbers = false;
@@ -101,7 +103,7 @@ const Hints = Module("hints", {
      * Display the current status to the user.
      */
     _updateStatusline: function () {
-        statusline.updateInputBuffer((hints.escNumbers ? mappings.getMapLeader() : "") + (this._hintNumber ? this._num2chars(this._hintNumber) : ""));
+        statusline.updateField("input", (hints.escNumbers ? mappings.getMapLeader() : "") + (this._hintNumber ? this._num2chars(this._hintNumber) : ""));
     },
 
     /**
@@ -130,35 +132,37 @@ const Hints = Module("hints", {
 
         let type = elem.type;
 
-        if (elem instanceof HTMLInputElement && /(submit|button|this._reset)/.test(type))
+        if (elem instanceof HTMLInputElement && /(submit|button|reset)/.test(type))
             return [elem.value, false];
         else {
-            for (let [, option] in Iterator(options["hintinputs"].split(","))) {
-                if (option == "value") {
-                    if (elem instanceof HTMLSelectElement) {
-                        if (elem.selectedIndex >= 0)
-                            return [elem.item(elem.selectedIndex).text.toLowerCase(), false];
-                    }
-                    else if (type == "image") {
-                        if (elem.alt)
-                            return [elem.alt.toLowerCase(), true];
-                    }
-                    else if (elem.value && type != "password") {
-                        // radio's and checkboxes often use internal ids as values - maybe make this an option too...
-                        if (! ((type == "radio" || type == "checkbox") && !isNaN(elem.value)))
-                            return [elem.value.toLowerCase(), (type == "radio" || type == "checkbox")];
-                    }
+            for (let option of options["hintinputs"].split(",")) {
+                switch (option) {
+                    case "value":
+                        if (elem instanceof HTMLSelectElement) {
+                            if (elem.selectedIndex >= 0)
+                                return [elem.item(elem.selectedIndex).text.toLowerCase(), false];
+                        }
+                        else if (type == "image") {
+                            if (elem.alt)
+                                return [elem.alt.toLowerCase(), true];
+                        }
+                        else if (elem.value && type != "password") {
+                            // radio's and checkboxes often use internal ids as values - maybe make this an option too...
+                            if (! ((type == "radio" || type == "checkbox") && !isNaN(elem.value)))
+                                return [elem.value.toLowerCase(), (type == "radio" || type == "checkbox")];
+                        }
+                        break;
+                    case "label":
+                        if (elem.id) {
+                            // TODO: (possibly) do some guess work for label-like objects
+                            let label = util.evaluateXPath(["label[@for=" + elem.id.quote() + "]"], doc).snapshotItem(0);
+                            if (label)
+                                return [label.textContent.toLowerCase(), true];
+                        }
+                        break;
+                    case "name":
+                        return [elem.name.toLowerCase(), true];
                 }
-                else if (option == "label") {
-                    if (elem.id) {
-                        // TODO: (possibly) do some guess work for label-like objects
-                        let label = util.evaluateXPath(["label[@for=" + elem.id.quote() + "]"], doc).snapshotItem(0);
-                        if (label)
-                            return [label.textContent.toLowerCase(), true];
-                    }
-                }
-                else if (option == "name")
-                    return [elem.name.toLowerCase(), true];
             }
         }
 
@@ -230,7 +234,11 @@ const Hints = Module("hints", {
 
     // the containing block offsets with respect to the viewport
     _getContainerOffsets: function (doc) {
-        let body = doc.body || doc.documentElement;
+        try {
+            var body = doc.body || doc.documentElement;
+        } catch (e) {
+            return [null, null];
+        }
         // TODO: getComputedStyle returns null for Facebook channel_iframe doc - probable Gecko bug.
         let style = util.computedStyle(body);
 
@@ -283,13 +291,13 @@ const Hints = Module("hints", {
             screen = {top: 0, left: 0, bottom: win.innerHeight, right: win.innerWidth};
 
         let doc = win.document;
-        let [offsetX, offsetY] = this._getContainerOffsets(doc);
 
-        let baseNodeAbsolute = util.xmlToDom(<span highlight="Hint"/>, doc);
+        let baseNodeAbsolute = util.xmlToDom(xml`<span highlight="Hint"/>`, doc);
 
         let res = util.evaluateXPath(this._hintMode.tags(), doc, null, true);
 
-        let fragment = util.xmlToDom(<div highlight="hints"/>, doc);
+        let fragment = util.xmlToDom(xml`<div highlight="hints" style="position:fixed;top:0;left:0px;"/>`, doc);
+        let pageHints = this._pageHints;
         let start = this._pageHints.length;
         let elem;
 
@@ -306,8 +314,8 @@ const Hints = Module("hints", {
 
             hint.span = baseNodeAbsolute.cloneNode(true);
 
-            let leftPos = Math.max((rect.left + offsetX), offsetX + screen.left);
-            let topPos =  Math.max((rect.top + offsetY), offsetY + screen.top);
+            let leftPos = rect.left > screen.left ? rect.left : screen.left;
+            let topPos =  rect.top  > screen.top  ? rect.top  : screen.top;
 
             if (elem instanceof HTMLAreaElement)
                 [leftPos, topPos] = that._getAreaOffset(elem, leftPos, topPos);
@@ -317,7 +325,7 @@ const Hints = Module("hints", {
 
             fragment.appendChild(hint.span);
 
-            that._pageHints.push(hint);
+            pageHints[pageHints.length] = hint;
         }
 
         while (elem = res.iterateNext()) {
@@ -349,7 +357,9 @@ const Hints = Module("hints", {
                 makeHint(elem);
         }
 
-        let body = doc.body || util.evaluateXPath(["body"], doc).snapshotItem(0);
+        let body = (doc instanceof XULDocument) ?
+                   doc.documentElement :
+                   doc.body || util.evaluateXPath(["body"], doc).snapshotItem(0);
         if (body) {
             body.appendChild(fragment);
             this._docs.push({ doc: doc, start: start, end: this._pageHints.length - 1 });
@@ -411,10 +421,19 @@ const Hints = Module("hints", {
         let validHint = this._hintMatcher(this._hintString.toLowerCase());
         let activeHint = this._hintNumber || 1;
         this._validHints = [];
+        this._tabNavigation = {};
+        let firstHint = null;
+        let lastHint = null;
+        let prevHint = null;
         let activeHintChars = this._num2chars(activeHint);
 
-        for (let [,{ doc: doc, start: start, end: end }] in Iterator(this._docs)) {
+        for (let { doc, start, end } of this._docs) {
+            if (!(doc instanceof Node))
+                continue;
+
             let [offsetX, offsetY] = this._getContainerOffsets(doc);
+            if (offsetX === null && offsetY === null)
+                continue;
 
         inner:
             for (let i in (util.interruptibleRange(start, end + 1, 500))) {
@@ -448,7 +467,7 @@ const Hints = Module("hints", {
                         if (!rect)
                             continue;
 
-                        hint.imgSpan = util.xmlToDom(<span highlight="Hint" liberator:class="HintImage" xmlns:liberator={NS}/>, doc);
+                        hint.imgSpan = util.xmlToDom(xml`<span highlight="Hint" liberator:class="HintImage" xmlns:liberator=${NS}/>`, doc);
                         hint.imgSpan.style.left = (rect.left + offsetX) + "px";
                         hint.imgSpan.style.top = (rect.top + offsetY) + "px";
                         hint.imgSpan.style.width = (rect.right - rect.left) + "px";
@@ -464,14 +483,26 @@ const Hints = Module("hints", {
                 else
                     this._setClass(hint.elem, activeHint == hintnum);
                 this._validHints.push(hint.elem);
+
+                // Set up tab navigation for valid hints
+                firstHint = (firstHint ) ? firstHint : hintnum ;
+                lastHint = hintnum;
+                prevHint = (prevHint) ? prevHint : hintnum;
+                this._tabNavigation[hintnum] = {prev: prevHint, next:prevHint};
+                this._tabNavigation[prevHint].next = hintnum;
+                prevHint = hintnum;
+
                 hintnum++;
             }
         }
 
+        this._tabNavigation[firstHint].prev = lastHint;
+        this._tabNavigation[lastHint].next = firstHint;
+
         if (config.browser.markupDocumentViewer.authorStyleDisabled) {
             let css = [];
             // FIXME: Broken for imgspans.
-            for (let [, { doc: doc }] in Iterator(this._docs)) {
+            for (let { doc } of this._docs) {
                 for (let elem in util.evaluateXPath(" {//*[@liberator:highlight and @number]", doc)) {
                     let group = elem.getAttributeNS(NS.uri, "highlight");
                     css.push(highlight.selector(group) + "[number=" + elem.getAttribute("number").quote() + "] { " + elem.style.cssText + " }");
@@ -494,7 +525,10 @@ const Hints = Module("hints", {
     _removeHints: function (timeout) {
         let firstElem = this._validHints[0] || null;
 
-        for (let [,{ doc: doc, start: start, end: end }] in Iterator(this._docs)) {
+        for (let { doc, start, end } of this._docs) {
+            if (!(doc instanceof Node))
+                continue;
+
             let result = util.evaluateXPath("//*[@liberator:highlight='hints']", doc, null, true);
             let hints = new Array();
             let elem;
@@ -762,7 +796,7 @@ const Hints = Module("hints", {
              */
             function stringsAtBeginningOfWords(strings, words, allowWordOverleaping) {
                 let strIdx = 0;
-                for (let [, word] in Iterator(words)) {
+                for (let word of words) {
                     if (word.length == 0)
                         continue;
 
@@ -865,6 +899,33 @@ const Hints = Module("hints", {
             this._checkUnique();
     },
 
+    /**
+     * start an extended hint mode
+     *
+     * @param {string} filter The filter to use.
+     * @param {Object} win The window in which we are showing hints.
+     */
+    startExtendedHint: function (filter, win) {
+        commandline.input(";",
+            function onSubmit (str) {
+                if (str in hints._hintModes)
+                    setTimeout(function(){ hints.show(str, filter, win); }, 0);
+            }, {
+                promptHighlight: "Normal",
+                completer: function (context) {
+                    context.compare = function () 0;
+                    context.completions = [[k, v.prompt] for ([k, v] in Iterator(hints._hintModes))];
+                },
+                onChange: function (str) {
+                    if (str in hints._hintModes) {
+                        hints.show(str, filter, win);
+                        if (commandline._completionList.visible())
+                            commandline._completionList.hide();
+                    }
+                },
+            });
+    },
+
     /**
      * Cancel all hinting.
      */
@@ -899,14 +960,11 @@ const Hints = Module("hints", {
                 this._hintNumber = 1;
 
             let oldId = this._hintNumber;
-            if (key == "<Tab>") {
-                if (++this._hintNumber > this._validHints.length)
-                    this._hintNumber = 1;
-            }
-            else {
-                if (--this._hintNumber < 1)
-                    this._hintNumber = this._validHints.length;
-            }
+            if (key == "<Tab>")
+                this._hintNumber = this._tabNavigation[this._hintNumber].next;
+            else
+                this._hintNumber = this._tabNavigation[this._hintNumber].prev;
+
             this._showActiveHint(this._hintNumber, oldId);
             this._updateStatusline();
             return;
@@ -1149,22 +1207,13 @@ const Hints = Module("hints", {
             "Start an extended hint mode",
             function (count) {
                 hints._extendedhintCount = count;
-                commandline.input(";", null,
-                    {
-                        promptHighlight: "Normal",
-                        completer: function (context) {
-                            context.compare = function () 0;
-                            context.completions = [[k, v.prompt] for ([k, v] in Iterator(hints._hintModes))];
-                        },
-                        onChange: function () { modes.reset(); },
-                        onCancel: function (arg) { arg && setTimeout(function () hints.show(arg), 0); }
-                    });
+                hints.startExtendedHint();
             }, { count: true });
     },
     options: function () {
         const DEFAULT_HINTTAGS =
             util.makeXPath(["input[not(@type='hidden')]", "a", "area", "iframe", "textarea", "button", "select"])
-                + " | //*[@onclick or @onmouseover or @onmousedown or @onmouseup or @oncommand or @role='link']"
+                + " | //*[@onclick or @onmouseover or @onmousedown or @onmouseup or @oncommand or @role='link'or @role='button' or @role='checkbox' or @role='combobox' or @role='listbox' or @role='listitem' or @role='menuitem' or @role='menuitemcheckbox' or @role='menuitemradio' or @role='option' or @role='radio' or @role='scrollbar' or @role='slider' or @role='spinbutton' or @role='tab' or @role='textbox' or @role='treeitem' or @tabindex]"
                 + (config.name == "Muttator" ?
                     " | //xhtml:div[@class='wrappedsender']/xhtml:div[contains(@class,'link')]" :
                     "");
