@@ -24,7 +24,7 @@ const Script = Class("Script", {
         this.__proto__ = plugins;
 
         // This belongs elsewhere
-        for (let [, dir] in Iterator(io.getRuntimeDirectories("plugin"))) {
+        for (let dir of io.getRuntimeDirectories("plugin")) {
             if (dir.contains(file, false))
                 plugins[this.NAME] = this;
         }
@@ -323,37 +323,68 @@ const IO = Module("io", {
     requires: ["config", "services"],
 
     init: function () {
-        this._processDir = services.get("directory").get("CurWorkD", Ci.nsIFile);
+        this._processDir = services.get("dirsvc").get("CurWorkD", Ci.nsIFile);
         this._cwd = this._processDir;
         this._oldcwd = null;
 
         this._lastRunCommand = ""; // updated whenever the users runs a command with :!
         this._scriptNames = [];
 
-        this.downloadListener = {
-            onDownloadStateChange: function (state, download) {
-                if (download.state == services.get("downloadManager").DOWNLOAD_FINISHED) {
-                    let url   = download.source.spec;
-                    let title = download.displayName;
-                    let file  = download.targetFile.path;
-                    let size  = download.size;
+        // XXX: nsIDownloadManager is deprecated on Firefox 26
+        // FIXME: need to listen to download state ? -- teramako
+        // FIXME: need to adapt to Download.jsm instead of nsIDownloadManager
+        if (services.get("vc").compare(Application.version, "26.0a1") < 0) {
+            this.downloadListener = {
+                onDownloadStateChange: function (state, download) {
+                    if (download.state == services.get("downloads").DOWNLOAD_FINISHED) {
+                        let url   = download.source.spec;
+                        let title = download.displayName;
+                        let file  = download.targetFile.path;
+                        let size  = download.size;
 
-                    liberator.echomsg("Download of " + title + " to " + file + " finished");
-                    autocommands.trigger("DownloadPost", { url: url, title: title, file: file, size: size });
-                }
-            },
-            onStateChange:    function () {},
-            onProgressChange: function () {},
-            onSecurityChange: function () {}
-        };
-
-        services.add("UUID",  "@mozilla.org/uuid-generator;1", Ci.nsIUUIDGenerator);
-
-        services.get("downloadManager").addListener(this.downloadListener);
+                        liberator.echomsg("Download of " + title + " to " + file + " finished");
+                        autocommands.trigger("DownloadPost", { url: url, title: title, file: file, size: size });
+                    }
+                },
+                onStateChange:    function () {},
+                onProgressChange: function () {},
+                onSecurityChange: function () {}
+            };
+            services.get("downloads").addListener(this.downloadListener);
+        } else {
+            let downloadListener = this.downloadListener = {
+                onDownloadChanged: function (download) {
+                    if (download.succeeded) {
+                        let {
+                            source: { url },
+                            target: {path: file},
+                            totalBytes: size,
+                        } = download;
+                        let title = File(file).leafName;
+                        liberator.echomsg("Download of " + title + " to " + file + " finished");
+                        autocommands.trigger("DownloadPost", { url: url, title: title, file: file, size: size });
+                    }
+                },
+            };
+            let {Downloads} = Cu.import("resource://gre/modules/Downloads.jsm", {});
+            Downloads.getList(Downloads.ALL)
+                .then(function (downloadList) {
+                    downloadList.addView(downloadListener);
+                });
+        }
     },
 
     destroy: function () {
-        services.get("downloadManager").removeListener(this.downloadListener);
+        if (services.get("vc").compare(Application.version, "26.0a1") < 0) {
+            services.get("downloads").removeListener(this.downloadListener);
+        } else {
+            let {Downloads} = Cu.import("resource://gre/modules/Downloads.jsm", {});
+            let downloadListener = this.downloadListener;
+            Downloads.getList(Downloads.ALL)
+                .then(function (downloadList) {
+                    downloadList.removeView(downloadListener);
+                });
+        }
         for (let [, plugin] in Iterator(plugins.contexts))
             if (plugin.onUnload)
                 plugin.onUnload();
@@ -485,7 +516,7 @@ const IO = Module("io", {
      * @returns {File}
      */
     createTempFile: function () {
-        let file = services.get("directory").get("TmpD", Ci.nsIFile);
+        let file = services.get("dirsvc").get("TmpD", Ci.nsIFile);
 
         file.append(config.tempFile);
         file.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
@@ -516,7 +547,7 @@ const IO = Module("io", {
                 dirs = [io.getCurrentDirectory().path].concat(dirs);
 
 lookup:
-            for (let [, dir] in Iterator(dirs)) {
+            for (let dir of dirs) {
                 file = File.joinPaths(dir, program);
                 try {
                     if (file.exists())
@@ -526,7 +557,7 @@ lookup:
                     // automatically try to add the executable path extensions on windows
                     if (liberator.has("Windows")) {
                         let extensions = services.get("environment").get("PATHEXT").split(";");
-                        for (let [, extension] in Iterator(extensions)) {
+                        for (let extension of extensions) {
                             file = File.joinPaths(dir, program + extension);
                             if (file.exists())
                                 break lookup;
@@ -580,8 +611,8 @@ lookup:
         liberator.log("Searching for \"" + paths.join(" ") + "\" in \"" + options["runtimepath"] + "\"");
 
         outer:
-        for (let [, dir] in Iterator(dirs)) {
-            for (let [, path] in Iterator(paths)) {
+        for (let dir of dirs) {
+            for (let path of paths) {
                 let file = File.joinPaths(dir, path);
 
                 if (file.exists() && file.isFile() && file.isReadable()) {
@@ -646,7 +677,7 @@ lookup:
                     let err = new Error();
                     for (let [k, v] in Iterator(e))
                         err[k] = v;
-                    err.echoerr = <>{file.path}:{e.lineNumber}: {e}</>;
+                    err.echoerr = xml`${file.path}:${e.lineNumber}: ${e}`;
                     throw err;
                 }
             }
@@ -683,7 +714,7 @@ lookup:
                         if (!command) {
                             let lineNumber = i + 1;
 
-                            liberator.echoerr("Error detected while processing: " + file.path, commandline.FORCE_MULTILINE);
+                            liberator.echoerr("Error detected while processing: " + file.path);
                             commandline.echo("line " + lineNumber + ":", commandline.HL_LINENR, commandline.APPEND_TO_MESSAGES);
                             liberator.echoerr("Not an editor command: " + line);
                         }
@@ -817,7 +848,7 @@ lookup:
      */
     get PATH_SEP() {
         delete this.PATH_SEP;
-        let f = services.get("directory").get("CurProcD", Ci.nsIFile);
+        let f = services.get("dirsvc").get("CurProcD", Ci.nsIFile);
         f.append("foo");
         return this.PATH_SEP = f.path.substr(f.parent.path.length, 1);
     }
@@ -846,7 +877,7 @@ lookup:
                     let dirs = File.getPathsFromPathList(options["cdpath"]);
                     let found = false;
 
-                    for (let [, dir] in Iterator(dirs)) {
+                    for (let dir of dirs) {
                         dir = File.joinPaths(dir, arg);
 
                         if (dir.exists() && dir.isDirectory() && dir.isReadable()) {
@@ -970,7 +1001,7 @@ lookup:
                 let output = io.system(arg);
 
                 commandline.command = "!" + arg;
-                commandline.echo(template.genericOutput("Command Output: " + arg, <span highlight="CmdOutput">{output}</span>));
+                commandline.echo(template.genericOutput("Command Output: " + arg, xml`<span highlight="CmdOutput">${String(output)}</span>`));
 
                 autocommands.trigger("ShellCmdPost", {});
             }, {
@@ -991,7 +1022,7 @@ lookup:
             context.anchored = false;
             context.generate = function () {
                 let names = util.Array(
-                    "more1 more2 more3 more4 more5 unicode".split(" ").map(function (key)
+                    "more1 more2 more3 more4 more5 static".split(" ").map(function (key)
                         options.getPref("intl.charsetmenu.browser." + key).split(', '))
                 ).flatten().uniq();
                 let bundle = document.getElementById("liberator-charset-bundle");
@@ -1026,7 +1057,7 @@ lookup:
             context.keys = {
                 text: !full ? "leafName" : function (f) dir + f.leafName,
                 description: function (f) f.isDirectory() ? "Directory" : "File",
-                isdir: function (f) f.isDirectory(),
+                isdir: function (f) f.exists() && f.isDirectory(),
                 icon: function (f) f.isDirectory() ? "resource://gre/res/html/folder.png"
                                                              : "moz-icon://" + f.leafName
             };
@@ -1062,7 +1093,7 @@ lookup:
                 let dirNames = services.get("environment").get("PATH").split(RegExp(liberator.has("Windows") ? ";" : ":"));
                 let commands = [];
 
-                for (let [, dirName] in Iterator(dirNames)) {
+                for (let dirName of dirNames) {
                     let dir = io.File(dirName);
                     if (dir.exists() && dir.isDirectory()) {
                         commands.push([[file.leafName, dir.path] for (file in dir.iterDirectory())
