@@ -40,9 +40,7 @@ const JavaScript = Module("javascript", {
             let orig = obj;
 
             function iterObj(obj, toplevel) {
-                function isXPCNativeWrapper(obj) isobject(obj) && XPCNativeWrapper.unwrap(obj) !== obj
-
-                if (isXPCNativeWrapper(obj)) {
+                if (Cu.isXrayWrapper(obj)) {
                     if (toplevel) {
                         yield {get wrappedJSObject() 0};
                     }
@@ -65,7 +63,7 @@ const JavaScript = Module("javascript", {
             }
 
             for (let obj in iterObj(orig, toplevel)) {
-                for (let [, k] in Iterator(Object.getOwnPropertyNames(obj))) {
+                for (let k of Object.getOwnPropertyNames(obj)) {
                     let name = "|" + k;
                     if (name in seen)
                         continue;
@@ -218,8 +216,19 @@ const JavaScript = Module("javascript", {
                 }
                 else if (this._c == this._last)
                     this._pop(this._c);
-            }
-            else {
+            } else if (this._last === "`") {
+                if (this._lastChar == "\\") { // Escape. Skip the next char, whatever it may be.
+                    this._c = "";
+                    this._i++;
+                }
+                else if (this._c == "`") {
+                    this._pop("`");
+                    this._pop("``");
+                } else if (this._c === "{" && this._lastChar === "$") {
+                    this._pop("`");
+                    this._push("{");
+                }
+            } else {
                 // A word character following a non-word character, or simply a non-word
                 // character. Start a new statement.
                 if (/[a-zA-Z_$]/.test(this._c) && !/[\w$]/.test(this._lastChar) || !/[\w\s$]/.test(this._c))
@@ -254,12 +263,20 @@ const JavaScript = Module("javascript", {
                 case ")": this._pop("("); break;
                 case "]": this._pop("["); break;
                 case "}": this._pop("{"); // Fallthrough
+                    if (this._last === "``") {
+                        this._push("`");
+                        break;
+                    }
                 case ";":
                     this._top.fullStatements.push(this._i);
                     break;
                 case ",":
                     this._top.comma.push(this._i);
                     break;
+                case "`":
+                    this._push("``");
+                    this._push("`");
+                    break;
                 }
 
                 if (/\S/.test(this._c))
@@ -297,7 +314,7 @@ const JavaScript = Module("javascript", {
         let prev = statement;
         let obj;
         let cacheKey;
-        for (let [, dot] in Iterator(this._get(frame).dots.concat(stop))) {
+        for (let dot of this._get(frame).dots.concat(stop)) {
             if (dot < statement)
                 continue;
             if (dot > stop || dot <= prev)
@@ -378,7 +395,7 @@ const JavaScript = Module("javascript", {
         }
         // TODO: Make this a generic completion helper function.
         let filter = key + (string || "");
-        for (let [, obj] in Iterator(objects)) {
+        for (let obj of objects) {
             this.context.fork(obj[1], this._top.offset, this, this._fill,
                 obj[0], obj[1], compl,
                 true, filter, last, key.length);
@@ -387,21 +404,21 @@ const JavaScript = Module("javascript", {
         if (orig)
             return;
 
-        for (let [, obj] in Iterator(objects)) {
+        for (let obj of objects) {
             let name = obj[1] + " (prototypes)";
             this.context.fork(name, this._top.offset, this, this._fill,
                 obj[0], name, function (a, b) compl(a, b, true),
                 true, filter, last, key.length);
         }
 
-        for (let [, obj] in Iterator(objects)) {
+        for (let obj of objects) {
             let name = obj[1] + " (substrings)";
             this.context.fork(name, this._top.offset, this, this._fill,
                 obj[0], name, compl,
                 false, filter, last, key.length);
         }
 
-        for (let [, obj] in Iterator(objects)) {
+        for (let obj of objects) {
             let name = obj[1] + " (prototype substrings)";
             this.context.fork(name, this._top.offset, this, this._fill,
                 obj[0], name, function (a, b) compl(a, b, true),
@@ -435,14 +452,14 @@ const JavaScript = Module("javascript", {
         }
 
         this.context.getCache("eval", Object);
-        this.context.getCache("evalContext", function () ({ __proto__: userContext }));
+        this.context.getCache("evalContext", function () Object.create(userContext));
 
         // Okay, have parse stack. Figure out what we're completing.
 
         // Find any complete statements that we can eval before we eval our object.
         // This allows for things like: let doc = window.content.document; let elem = doc.createElement...; elem.<Tab>
         let prev = 0;
-        for (let [, v] in Iterator(this._get(0).fullStatements)) {
+        for (let v of this._get(0).fullStatements) {
             let key = this._str.substring(prev, v + 1);
             if (this._checkFunction(prev, v, key))
                 return null;
@@ -452,7 +469,7 @@ const JavaScript = Module("javascript", {
 
         // In a string. Check if we're dereferencing an object.
         // Otherwise, do nothing.
-        if (this._last == "'" || this._last == '"') {
+        if (this._last == "'" || this._last == '"' || this._last == "`") {
             //
             // str = "foo[bar + 'baz"
             // obj = "foo"
@@ -510,15 +527,17 @@ const JavaScript = Module("javascript", {
                 // Split up the arguments
                 let prev = this._get(-2).offset;
                 let args = [];
-                for (let [i, idx] in Iterator(this._get(-2).comma)) {
+                let i = 0;
+                for (let idx of this._get(-2).comma) {
                     let arg = this._str.substring(prev + 1, idx);
                     prev = idx;
                     util.memoize(args, i, function () self.eval(arg));
+                    ++i;
                 }
                 let key = this._getKey();
                 args.push(key + string);
 
-                compl = function (context, obj) {
+                var compl = function (context, obj) {
                     let res = completer.call(self, context, func, obj, args);
                     if (res)
                         context.completions = res;
@@ -597,7 +616,7 @@ const JavaScript = Module("javascript", {
      */
     setCompleter: function (funcs, completers) {
         funcs = Array.concat(funcs);
-        for (let [, func] in Iterator(funcs)) {
+        for (let func of funcs) {
             func.liberatorCompleter = function (context, func, obj, args) {
                 let completer = completers[args.length - 1];
                 if (!completer)
@@ -615,5 +634,8 @@ const JavaScript = Module("javascript", {
         options.add(["inspectcontentobjects"],
             "Allow completion of JavaScript objects coming from web content. POSSIBLY INSECURE!",
             "boolean", false);
+        options.add(["expandtemplate"],
+            "Expand TemplateLiteral",
+            "boolean", !("XMLList" in window));
     }
 })
