@@ -55,8 +55,6 @@ const Liberator = Module("liberator", {
 
     init: function () {
         window.liberator = this;
-        window.liberator = this;
-        modules.liberator = this;
         this.observers = {};
         this.modules = modules;
 
@@ -66,12 +64,12 @@ const Liberator = Module("liberator", {
         // current profile. These will differ if the current process was run
         // without explicitly selecting a profile.
         /** @property {string} The name of the current user profile. */
-        this.profileName = services.get("directory").get("ProfD", Ci.nsIFile).leafName.replace(/^.+?\./, "");
+        this.profileName = services.get("dirsvc").get("ProfD", Ci.nsIFile).leafName.replace(/^.+?\./, "");
 
         let platform = Liberator.getPlatformFeature()
-        config.features.push(platform);
+        config.features.add(platform);
         if (/^Win(32|64)$/.test(platform))
-            config.features.push('Windows');
+            config.features.add('Windows');
         
         if (AddonManager) {
             let self = this;
@@ -125,12 +123,14 @@ const Liberator = Module("liberator", {
     NEW_TAB: [],
     NEW_BACKGROUND_TAB: [],
     NEW_WINDOW: [],
+    NEW_PRIVATE_WINDOW: [],
 
     forceNewTab: false,
     forceNewWindow: false,
+    forceNewPrivateWindow: false,
 
     /** @property {string} The Liberator version string. */
-    version: "3.3 (created: 2011/11/13 17:32:12)", // these VERSION and DATE tokens are replaced by the Makefile
+    version: "3.8.2 (created: 2014/02/13 23:52:43)", // these VERSION and DATE tokens are replaced by the Makefile
 
     /**
      * @property {Object} The map of command-line options. These are
@@ -165,7 +165,7 @@ const Liberator = Module("liberator", {
     // TODO: "zoom": if the zoom value of the current buffer changed
     triggerObserver: function (type) {
         let args = Array.slice(arguments, 1);
-        for (let [, func] in Iterator(this.observers[type] || []))
+        for (let func of this.observers[type] || [])
             func.apply(null, args);
     },
 
@@ -287,21 +287,20 @@ const Liberator = Module("liberator", {
 
             // For errors, also print the stack trace to our :messages list
             if (str instanceof Error) {
-                let stackTrace = <></>;
+                let stackTrace = xml``;
                 let stackItems = new Error().stack.split('\n');
                 // ignore the first element intenationally!
-                for (let i = 1; i < stackItems.length; i++) {
-                    let stackItem = stackItems[i];
+                stackTrace = template.map2(xml, stackItems.slice(1), function (stackItema) {
                     let atIndex = stackItem.lastIndexOf("@");
-                    if (atIndex < 0)
-                        continue;
+                    if (n === 0 || atIndex < 0)
+                        return "";
                     let stackLocation = stackItem.substring(atIndex + 1);
                     let stackArguments = stackItem.substring(0, atIndex);
                     if (stackArguments)
                         stackArguments = " - " + stackArguments;
 
-                    stackTrace += <><span style="font-weight: normal">&#xa0;&#xa0;at </span>{stackLocation}<span style="font-weight: normal">{stackArguments}</span><br/></>;
-                }
+                    return xml`<span style="font-weight: normal">&#xa0;&#xa0;xxx at </span>${stackLocation}<span style="font-weight: normal">${stackArguments}</span><br/>`;
+                });
                 commandline.messages.add({str: stackTrace, highlight: commandline.HL_ERRORMSG});
             }
         } catch (e) {
@@ -333,15 +332,25 @@ const Liberator = Module("liberator", {
      *     should be loaded.
      */
     loadScript: function (uri, context) {
-        XML.ignoreWhiteSpace = false;
-        XML.prettyPrinting = false;
-        services.get("subscriptLoader").loadSubScript(uri, context, "UTF-8");
+        if (options.expandtemplate) {
+            var prefix = "liberator://template/";
+            if (uri.lastIndexOf(prefix, 0) === -1)
+                uri = prefix + uri;
+        }
+        services.get("scriptloader").loadSubScript(uri, context, "UTF-8");
     },
 
     eval: function (str, context) {
         try {
             if (!context)
                 context = userContext;
+
+            if (options.expandtemplate) {
+                var obj = new Object;
+                Cu.import("resource://liberator/template.js", obj);
+                str = obj.convert(str);
+            }
+
             context[EVAL_ERROR] = null;
             context[EVAL_STRING] = str;
             context[EVAL_RESULT] = null;
@@ -370,24 +379,11 @@ const Liberator = Module("liberator", {
     //       I agree, the name is confusing, and so is the
     //           description --Kris
     evalExpression: function (string) {
-        string = string.toString().replace(/^\s*/, "").replace(/\s*$/, "");
+        string = string.toString().trim();
 
-        let matches = string.match(/^&(\w+)/);
-        if (matches) {
-            let opt = this.options.get(matches[1]);
-
-            liberator.assert(opt, "Unknown option: " + matches[1]);
-
-            let type = opt.type;
-            let value = opt.getter();
-
-            if (type != "boolean" && type != "number")
-                value = value.toString();
-
-            return value;
-        }
+        let matches = null;
         // String
-        else if ((matches = string.match(/^(['"])([^\1]*?[^\\]?)\1/))) {
+        if ((matches = string.match(/^(['"])([^\1]*?[^\\]?)\1/))) {
             return matches[2].toString();
         }
         // Number
@@ -448,7 +444,7 @@ const Liberator = Module("liberator", {
      *     element.
      */
     focusContent: function (clearFocusedElement) {
-        if (window != services.get("windowWatcher").activeWindow)
+        if (window != services.get("ww").activeWindow)
             return;
 
         let elem = config.mainWidget || window.content;
@@ -456,7 +452,7 @@ const Liberator = Module("liberator", {
         try {
             if (this.has("tabs")) {
                 // select top most frame in a frameset
-                let frame = tabs.localStore.focusedFrame;
+                let frame = buffer.localStore.focusedFrame;
                 if (frame && frame.top == window.content)
                     elem = frame;
             }
@@ -475,7 +471,7 @@ const Liberator = Module("liberator", {
      * @param {string} feature The feature name.
      * @returns {boolean}
      */
-    has: function (feature) config.features.indexOf(feature) >= 0,
+    has: function (feature) config.features.has(feature),
 
     /**
      * Returns whether the host application has the specified extension
@@ -504,7 +500,7 @@ const Liberator = Module("liberator", {
 
         function format(item) item.description + "#" + encodeURIComponent(item.text);
 
-        for (let [i, item] in Iterator(items)) {
+        for (let item of items) {
             if (item.text == topic)
                 return format(item);
             else if (!partialMatch && topic)
@@ -539,7 +535,7 @@ const Liberator = Module("liberator", {
         // Find help and overlay files with the given name.
         function findHelpFile(file) {
             let result = [];
-            for (let [, namespace] in Iterator(namespaces)) {
+            for (let namespace of namespaces) {
                 let url = ["chrome://", namespace, "/locale/", file, ".xml"].join("");
                 let res = util.httpGet(url);
                 if (res) {
@@ -577,42 +573,42 @@ const Liberator = Module("liberator", {
         });
 
         // Process plugin help entries.
-        XML.ignoreWhiteSpace = false;
-        XML.prettyPrinting = false;
-        XML.prettyPrinting = true; // Should be false, but ignoreWhiteSpace=false doesn't work correctly. This is the lesser evil.
-        XML.prettyIndent = 4;
-
         let lang = options.getPref("general.useragent.locale", "en-US");
-        function chooseByLang(elems) {
-            if (!elems)
-                return null;
-            function get(lang) {
-                let i = elems.length();
-                while (i-- > 0){
-                    if ((elems[i].@lang).toString() == lang)
-                        return elems[i];
-                }
+        const ps = new DOMParser;
+        const encoder = Cc["@mozilla.org/layout/documentEncoder;1?type=text/xml"].getService(Ci.nsIDocumentEncoder);
+        encoder.init(document, "text/xml", 0);
+        body = xml.map([con for ([,con] in Iterator(plugins.contexts))], function (context) {
+            try { // debug
+            var info = context.INFO;
+            if (!info) return "";
+            var div = ps.parseFromString(xml`<div xmlns=${XHTML}>${info}</div>`, "text/xml").documentElement;
+            var list = div.querySelectorAll("plugin");
+            var res = {};
+            for (var i = 0, j = list.length; i < j; i++) {
+                var info = list[i];
+                var value = info.getAttribute("lang") || "";
+                res[i] = res[value] = info;
             }
-            elems = elems.(function::nodeKind() == "element");
-            return get(lang) || get(lang.split("-", 2).shift()) || get("") || get("en-US") || get("en") || elems[0] || elems;
-        }
-        let body = XML();
-        for (let [, context] in Iterator(plugins.contexts)) {
-            if (context.INFO instanceof XML) {
-                let info = chooseByLang(context.INFO);
-                body += <h2 xmlns={NS.uri} tag={info.@name + '-plugin'}>{info.@summary}</h2> + info;
+            var info = res[lang] || res[lang.split("-", 2).shift()] || res[0];
+            if (!info) return "";
+            encoder.setNode(info);
+            return xml`<h2 xmlns=${NS.uri} tag=${info.getAttribute("name") + '-plugin'}>${
+                info.getAttribute("summary")}</h2>${xml.raw`${encoder.encodeToString()}`}`;
+            } catch (ex) {
+                Cu.reportError(ex);
+                alert(ex);
             }
-        }
+        });
 
         let help = '<?xml version="1.0"?>\n' +
                    '<?xml-stylesheet type="text/xsl" href="chrome://liberator/content/help.xsl"?>\n' +
                    '<!DOCTYPE document SYSTEM "chrome://liberator/content/liberator.dtd">' +
-            <document xmlns={NS}
-                name="plugins" title={config.name + " Plugins"}>
+            xml`<document xmlns=${NS}
+                name="plugins" title=${config.name + " Plugins"}>
                 <h1 tag="using-plugins">Using Plugins</h1>
 
-                {body}
-            </document>.toXMLString();
+                ${body}
+            </document>`.toString();
         fileMap["plugins"] = function () ['text/xml;charset=UTF-8', help];
 
         addTags("plugins", util.httpGet("liberator://help/plugins").responseXML);
@@ -679,7 +675,7 @@ const Liberator = Module("liberator", {
         }
 
         liberator.log('Searching for "plugin/**/*.{js,vimp}" in "'
-                            + [dir.path.replace(/.plugin$/, "") for ([, dir] in Iterator(dirs))].join(",") + '"');
+                            + [dir.path.replace(/.plugin$/, "") for (dir of dirs)].join(",") + '"');
 
         dirs.forEach(function (dir) {
             liberator.log("Searching for \"" + (dir.path + "/**/*.{js,vimp}") + "\"", 3);
@@ -694,7 +690,9 @@ const Liberator = Module("liberator", {
      */
     log: function (msg) {
         if (typeof msg == "object")
-            msg = util.objectToString(msg, false);
+            msg = Cc["@mozilla.org/feed-unescapehtml;1"]
+                    .getService(Ci.nsIScriptableUnescapeHTML)
+                    .unescape(util.objectToString(msg, false).value);
 
         services.get("console").logStringMessage(config.name.toLowerCase() + ": " + msg);
     },
@@ -756,6 +754,8 @@ const Liberator = Module("liberator", {
             where = liberator.NEW_TAB;
         else if (liberator.forceNewWindow)
             where = liberator.NEW_WINDOW;
+        else if (liberator.forceNewPrivateWindow)
+            where = liberator.NEW_PRIVATE_WINDOW;
 
         if ("from" in params && liberator.has("tabs")) {
             if (!('where' in params) && options["newtab"] && options.get("newtab").has("all", params.from))
@@ -772,10 +772,22 @@ const Liberator = Module("liberator", {
             return;
 
         let browser = config.browser;
+        let urlTasks = [];
         function open(urls, where) {
+            if (!browser) {
+                urlTasks.push(urls);
+                return;
+            }
+
             try {
-                let url = Array.concat(urls)[0];
-                let postdata = Array.concat(urls)[1];
+                let url = "", postdata;
+                if (typeof urls === "string")
+                    url = urls;
+                else
+                    [url, postdata] = Array.concat(urls);
+
+                if (!url)
+                    url = window.BROWSER_NEW_TAB_URL || "about:blank";
 
                 // decide where to load the first url
                 switch (where) {
@@ -796,18 +808,34 @@ const Liberator = Module("liberator", {
                     });
                     break;
 
+                case liberator.NEW_PRIVATE_WINDOW:
                 case liberator.NEW_WINDOW:
-                    window.open();
-                    let win = services.get("windowMediator").getMostRecentWindow("navigator:browser");
-                    win.loadURI(url, null, postdata);
-                    browser = win.getBrowser();
+                    let sa = Cc["@mozilla.org/supports-array;1"].createInstance(Ci.nsISupportsArray);
+                    let wuri = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
+                    wuri.data = url;
+                    sa.AppendElement(wuri);
+                    sa.AppendElement(null); // charset
+                    sa.AppendElement(null); // referrerURI
+                    sa.AppendElement(postdata);
+                    sa.AppendElement(null); // allowThirdPartyFixup
+
+                    let features = "chrome,dialog=no,all" + (where === liberator.NEW_PRIVATE_WINDOW ? ",private" : "");
+                    let win = services.get("ww").openWindow(window, "chrome://browser/content/browser.xul", null, features, sa);
+                    browser = null;
+                    win.addEventListener("load", function onload(aEvent) {
+                        win.removeEventListener("load", onload, false);
+                        browser = win.getBrowser();
+                        for (let url of urlTasks)
+                            open(url, liberator.NEW_BACKGROUND_TAB);
+                        urlTasks = [];
+                    }, false);
                     break;
                 }
             }
             catch (e) {}
         }
 
-        for (let [, url] in Iterator(urls)) {
+        for (let url of urls) {
             open(url, where);
             where = liberator.NEW_BACKGROUND_TAB;
         }
@@ -837,7 +865,7 @@ const Liberator = Module("liberator", {
             options.setPref("browser.startup.page", 1); // start with default homepage session
 
         if (force)
-            services.get("appStartup").quit(Ci.nsIAppStartup.eForceQuit);
+            services.get("startup").quit(Ci.nsIAppStartup.eForceQuit);
         else
             window.goQuitApplication();
     },
@@ -915,23 +943,23 @@ const Liberator = Module("liberator", {
     restart: function () {
         // notify all windows that an application quit has been requested.
         var cancelQuit = Cc["@mozilla.org/supports-PRBool;1"].createInstance(Ci.nsISupportsPRBool);
-        services.get("observer").notifyObservers(cancelQuit, "quit-application-requested", null);
+        services.get("obs").notifyObservers(cancelQuit, "quit-application-requested", null);
 
         // something aborted the quit process.
         if (cancelQuit.data)
             return;
 
         // notify all windows that an application quit has been granted.
-        services.get("observer").notifyObservers(null, "quit-application-granted", null);
+        services.get("obs").notifyObservers(null, "quit-application-granted", null);
 
         // enumerate all windows and call shutdown handlers
-        let windows = services.get("windowMediator").getEnumerator(null);
+        let windows = services.get("wm").getEnumerator(null);
         while (windows.hasMoreElements()) {
             let win = windows.getNext();
             if (("tryToClose" in win) && !win.tryToClose())
                 return;
         }
-        services.get("appStartup").quit(Ci.nsIAppStartup.eRestart | Ci.nsIAppStartup.eAttemptQuit);
+        services.get("startup").quit(Ci.nsIAppStartup.eRestart | Ci.nsIAppStartup.eAttemptQuit);
     },
 
     /**
@@ -1010,7 +1038,7 @@ const Liberator = Module("liberator", {
      */
     get windows() {
         let windows = [];
-        let enumerator = services.get("windowMediator").getEnumerator("navigator:browser");
+        let enumerator = services.get("wm").getEnumerator("navigator:browser");
         while (enumerator.hasMoreElements())
             windows.push(enumerator.getNext());
 
@@ -1030,7 +1058,7 @@ const Liberator = Module("liberator", {
     // TODO: move this
     getMenuItems: function () {
         function addChildren(node, parent) {
-            for (let [, item] in Iterator(node.childNodes)) {
+            for (let item of node.childNodes) {
                 if (item.childNodes.length == 0 && item.localName == "menuitem"
                     && !/rdf:http:/.test(item.getAttribute("label"))) { // FIXME
                     item.fullMenuPath = parent + item.getAttribute("label");
@@ -1118,26 +1146,26 @@ const Liberator = Module("liberator", {
             {
                 setter: function (value) {
                     let win = document.documentElement;
-                    function updateTitle(old, current) {
-                        document.title = document.title.replace(RegExp("(.*)" + util.escapeRegex(old)), "$1" + current);
-                    }
-
-                    if (services.get("privateBrowsing")) {
+                    if (liberator.has("privatebrowsing")) {
                         let oldValue = win.getAttribute("titlemodifier_normal");
                         let suffix = win.getAttribute("titlemodifier_privatebrowsing").substr(oldValue.length);
 
                         win.setAttribute("titlemodifier_normal", value);
                         win.setAttribute("titlemodifier_privatebrowsing", value + suffix);
 
-                        if (services.get("privateBrowsing").privateBrowsingEnabled) {
-                            updateTitle(oldValue + suffix, value + suffix);
-                            return value;
+                        if (window.QueryInterface(Ci.nsIInterfaceRequestor)
+                                  .getInterface(Ci.nsIWebNavigation)
+                                  .QueryInterface(Ci.nsILoadContext)
+                                  .usePrivateBrowsing) {
+                            win.setAttribute("titlemodifier", value + suffix);
                         }
+                        else
+                            win.setAttribute("titlemodifier", value);
                     }
+                    else
+                        win.setAttribute("titlemodifier", value);
 
-                    updateTitle(win.getAttribute("titlemodifier"), value);
-                    win.setAttribute("titlemodifier", value);
-
+                    config.updateTitlebar();
                     return value;
                 }
             });
@@ -1152,38 +1180,37 @@ const Liberator = Module("liberator", {
                     // Used in order to avoid multiple collapse/uncollapse actions
                     // for values like :set gui=none,tabs
                     let actions = {};
-                    for (let [, action] in Iterator(this.parseValues(values))) {
-                        if (action == "all" || action == "none") {
+                    for (let action of this.parseValues(values)) {
+                        if (action === "all" || action === "none") {
                             for (let [name, toolbar] in Iterator(toolbars)) {
-                                let ids = toolbar[0] || [];
-                                ids.forEach(function (id) actions[id] = action == "none");
+                                for (let id of toolbar[0] || [])
+                                    actions[id] = action === "none";
                             }
                         } else {
                             let toolbarName = action.replace(/^(no|inv)/, "");
                             let toolbar = toolbars[toolbarName];
                             if (toolbar) {
-                                let ids = toolbar[0] || [];
-                                ids.forEach(function (id) {
+                                for (let id of toolbar[0] || []) {
                                     let elem = document.getElementById(id);
                                     if (!elem)
-                                        return;
+                                        continue;
 
                                     let collapsed = false;
-                                    if (action.indexOf("no") == 0)
+                                    if (action.startsWith("no"))
                                         collapsed = true;
-                                    else if (action.indexOf("inv") == 0) {
-                                        if (typeof(actions[id]) == "boolean")
+                                    else if (action.startsWith("inv")) {
+                                        if (typeof(actions[id]) === "boolean")
                                             collapsed = !actions[id];
                                         else {
-                                            let hidingAttribute = elem.getAttribute("type") == "menubar" ? "autohide" : "collapsed";
-                                            collapsed = !(elem.getAttribute(hidingAttribute) == "true");
+                                            let hidingAttribute = elem.getAttribute("type") === "menubar" ? "autohide" : "collapsed";
+                                            collapsed = !(elem.getAttribute(hidingAttribute) === "true");
                                         }
                                     }
                                     else
                                         collapsed = false;
                                     
                                     actions[id] = collapsed; // add the action, or change an existing action
-                                });
+                                }
                             }
                         }
                     }
@@ -1285,10 +1312,7 @@ const Liberator = Module("liberator", {
     commands: function () {
         commands.add(["addo[ns]"],
             "Manage available Extensions and Themes",
-            function () {
-                liberator.open("chrome://mozapps/content/extensions/extensions.xul",
-                    { from: "addons" });
-            },
+            function () { liberator.open("about:addons", { from: "addons" }); },
             { argCount: "0" });
 
         commands.add(["beep"],
@@ -1305,7 +1329,7 @@ const Liberator = Module("liberator", {
                     // TODO: why are these sorts of properties arrays? --djk
                     let dialogs = config.dialogs;
 
-                    for (let [, dialog] in Iterator(dialogs)) {
+                    for (let dialog of dialogs) {
                         if (util.compareIgnoreCase(arg, dialog[0]) == 0) {
                             dialog[2]();
                             return;
@@ -1469,8 +1493,8 @@ const Liberator = Module("liberator", {
                         ["Name", "Version", "Status", "Description"],
                         ([template.icon(e, e.name),
                           e.version,
-                          e.enabled ? <span highlight="Enabled">enabled</span>
-                                    : <span highlight="Disabled">disabled</span>,
+                          e.enabled ? xml`<span highlight="Enabled">enabled</span>`
+                                    : xml`<span highlight="Disabled">disabled</span>`,
                           e.description] for ([, e] in Iterator(extensions)))
                     );
 
@@ -1605,11 +1629,11 @@ const Liberator = Module("liberator", {
                             totalUnits = "msec";
 
                         let str = template.genericOutput("Code execution summary",
-                                <table>
-                                    <tr><td>Executed:</td><td align="right"><span class="times-executed">{count}</span></td><td>times</td></tr>
-                                    <tr><td>Average time:</td><td align="right"><span class="time-average">{each.toFixed(2)}</span></td><td>{eachUnits}</td></tr>
-                                    <tr><td>Total time:</td><td align="right"><span class="time-total">{total.toFixed(2)}</span></td><td>{totalUnits}</td></tr>
-                                </table>);
+                                xml`<table>
+                                    <tr><td>Executed:</td><td align="right"><span class="times-executed">${count}</span></td><td>times</td></tr>
+                                    <tr><td>Average time:</td><td align="right"><span class="time-average">${each.toFixed(2)}</span></td><td>${eachUnits}</td></tr>
+                                    <tr><td>Total time:</td><td align="right"><span class="time-total">${total.toFixed(2)}</span></td><td>${totalUnits}</td></tr>
+                                </table>`);
                         commandline.echo(str, commandline.HL_NORMAL, commandline.FORCE_MULTILINE);
                     }
                     else {
@@ -1684,9 +1708,9 @@ const Liberator = Module("liberator", {
             "List all commands, mappings and options with a short description",
             function (args) {
                 let usage = {
-                    mappings: function() template.table("Mappings", [[item.name || item.names[0], item.description] for (item in mappings)].sort()),
-                    commands: function() template.table("Commands", [[item.name || item.names[0], item.description] for (item in commands)]),
-                    options:  function() template.table("Options",  [[item.name || item.names[0], item.description] for (item in options)])
+                    mappings: function() template.table2(xml, "Mappings", [[item.name || item.names[0], item.description] for (item in mappings)].sort()),
+                    commands: function() template.table2(xml, "Commands", [[item.name || item.names[0], item.description] for (item in commands)]),
+                    options:  function() template.table2(xml, "Options",  [[item.name || item.names[0], item.description] for (item in options)])
                 }
 
                 if (args[0] && !usage[args[0]])
@@ -1695,7 +1719,7 @@ const Liberator = Module("liberator", {
                 if (args[0])
                     var usage = template.genericOutput(config.name + " Usage", usage[args[0]]());
                 else
-                    var usage = template.genericOutput(config.name + " Usage", usage["mappings"]() + <br/> + usage["commands"]() + <br/> + usage["options"]());
+                    var usage = template.genericOutput(config.name + " Usage", xml`${ usage["mappings"]() }<br/>${ usage["commands"]() }<br/>${ usage["options"]()}`);
                 liberator.echo(usage, commandline.FORCE_MULTILINE);
             }, {
                 argCount: "?",
