@@ -437,7 +437,7 @@ const Options = Module("options", {
         this._optionHash = {};
         this._prefContexts = [];
 
-        for (let [, pref] in Iterator(this.allPrefs(Options.OLD_SAVED))) {
+        for (let pref of this.allPrefs(Options.OLD_SAVED)) {
             let saved = Options.SAVED + pref.substr(Options.OLD_SAVED.length)
             if (!this.getPref(saved))
                 this.setPref(saved, this.getPref(pref));
@@ -478,14 +478,14 @@ const Options = Module("options", {
     /** @property {Iterator(Option)} @private */
     __iterator__: function () {
         let sorted = [o for ([i, o] in Iterator(this._optionHash))].sort(function (a, b) String.localeCompare(a.name, b.name));
-        return (v for ([k, v] in Iterator(sorted)));
+        return (v for (v of sorted));
     },
 
     /** @property {Object} Observes preference value changes. */
     prefObserver: {
         register: function () {
             // better way to monitor all changes?
-            this._branch = services.get("pref").getBranch("").QueryInterface(Ci.nsIPrefBranch2);
+            this._branch = services.get("prefs").getBranch("").QueryInterface(Ci.nsIPrefBranch2);
             this._branch.addObserver("", this, false);
         },
 
@@ -550,7 +550,7 @@ const Options = Module("options", {
      * @param {string} branch The branch in which to search preferences.
      *     @default ""
      */
-    allPrefs: function (branch) services.get("pref").getChildList(branch || "", { value: 0 }),
+    allPrefs: function (branch) services.get("prefs").getChildList(branch || "", { value: 0 }),
 
     /**
      * Returns the option with <b>name</b> in the specified <b>scope</b>.
@@ -567,7 +567,7 @@ const Options = Module("options", {
         if (name in this._optionHash)
             return (this._optionHash[name].scope & scope) && this._optionHash[name];
 
-        for (let opt in Iterator(options)) {
+        for (let opt in options) {
             if (opt.hasName(name))
                 return (opt.scope & scope) && opt;
         }
@@ -589,13 +589,13 @@ const Options = Module("options", {
             scope = Option.SCOPE_BOTH;
 
         function opts(opt) {
-            for (let opt in Iterator(options)) {
+            for (let opt in options) {
                 let option = {
                     isDefault: opt.value == opt.defaultValue,
                     name:      opt.name,
                     default:   opt.defaultValue,
                     pre:       "\u00a0\u00a0", // Unicode nonbreaking space.
-                    value:     <></>
+                    value:     xml``
                 };
 
                 if (onlyNonDefault && option.isDefault)
@@ -609,7 +609,7 @@ const Options = Module("options", {
                     option.default = (option.default ? "" : "no") + opt.name;
                 }
                 else
-                    option.value = <>={template.highlight(opt.value)}</>;
+                    option.value = xml`=${template.highlight(opt.value)}`;
                 yield option;
             }
         };
@@ -635,8 +635,8 @@ const Options = Module("options", {
         let prefArray = options.allPrefs();
         prefArray.sort();
         function prefs() {
-            for (let [, pref] in Iterator(prefArray)) {
-                let userValue = services.get("pref").prefHasUserValue(pref);
+            for (let pref of prefArray) {
+                let userValue = services.get("prefs").prefHasUserValue(pref);
                 if (onlyNonDefault && !userValue || pref.indexOf(filter) == -1)
                     continue;
 
@@ -645,7 +645,7 @@ const Options = Module("options", {
                 let option = {
                     isDefault: !userValue,
                     default:   options._loadPreference(pref, null, true),
-                    value:     <>={template.highlight(value, true, 100)}</>,
+                    value:     xml`${template.highlight(value, true, 100)}`,
                     name:      pref,
                     pre:       "\u00a0\u00a0" // Unicode nonbreaking space.
                 };
@@ -775,7 +775,7 @@ const Options = Module("options", {
      */
     resetPref: function (name) {
         try {
-            services.get("pref").clearUserPref(name);
+            services.get("prefs").clearUserPref(name);
         }
         catch (e) {
             // ignore - thrown if not a user set value
@@ -788,7 +788,7 @@ const Options = Module("options", {
      * @param {string} name The preference name.
      */
     invertPref: function (name) {
-        if (services.get("pref").getPrefType(name) == Ci.nsIPrefBranch.PREF_BOOL)
+        if (services.get("prefs").getPrefType(name) == Ci.nsIPrefBranch.PREF_BOOL)
             this.setPref(name, !this.getPref(name));
         else
             liberator.echoerr("Trailing characters: " + name + "!");
@@ -841,13 +841,13 @@ const Options = Module("options", {
                 this._prefContexts[this._prefContexts.length - 1][name] = val;
         }
 
-        let type = services.get("pref").getPrefType(name);
+        let type = services.get("prefs").getPrefType(name);
         switch (typeof value) {
         case "string":
             if (type == Ci.nsIPrefBranch.PREF_INVALID || type == Ci.nsIPrefBranch.PREF_STRING) {
                 let supportString = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
                 supportString.data = value;
-                services.get("pref").setComplexValue(name, Ci.nsISupportsString, supportString);
+                services.get("prefs").setComplexValue(name, Ci.nsISupportsString, supportString);
             }
             else if (type == Ci.nsIPrefBranch.PREF_INT)
                 liberator.echoerr("Number required after =: " + name + "=" + value);
@@ -856,13 +856,13 @@ const Options = Module("options", {
             break;
         case "number":
             if (type == Ci.nsIPrefBranch.PREF_INVALID || type == Ci.nsIPrefBranch.PREF_INT)
-                services.get("pref").setIntPref(name, value);
+                services.get("prefs").setIntPref(name, value);
             else
                 liberator.echoerr("Invalid argument: " + name + "=" + value);
             break;
         case "boolean":
             if (type == Ci.nsIPrefBranch.PREF_INVALID || type == Ci.nsIPrefBranch.PREF_BOOL)
-                services.get("pref").setBoolPref(name, value);
+                services.get("prefs").setBoolPref(name, value);
             else if (type == Ci.nsIPrefBranch.PREF_INT)
                 liberator.echoerr("Number required after =: " + name + "=" + value);
             else
@@ -878,14 +878,14 @@ const Options = Module("options", {
         if (forcedDefault != null)  // this argument sets defaults for non-user settable options (like extensions.history.comp_history)
             defaultValue = forcedDefault;
 
-        let branch = defaultBranch ? services.get("pref").getDefaultBranch("") : services.get("pref");
-        let type = services.get("pref").getPrefType(name);
+        let branch = defaultBranch ? services.get("prefs").getDefaultBranch("") : services.get("prefs");
+        let type = services.get("prefs").getPrefType(name);
         try {
             switch (type) {
             case Ci.nsIPrefBranch.PREF_STRING:
                 let value = branch.getComplexValue(name, Ci.nsISupportsString).data;
                 // try in case it's a localized string (will throw an exception if not)
-                if (!services.get("pref").prefIsLocked(name) && !services.get("pref").prefHasUserValue(name) &&
+                if (!services.get("prefs").prefIsLocked(name) && !services.get("prefs").prefHasUserValue(name) &&
                     RegExp("chrome://.+/locale/.+\\.properties").test(value))
                         value = branch.getComplexValue(name, Ci.nsIPrefLocalizedString).data;
                 return value;
@@ -1004,7 +1004,7 @@ const Options = Module("options", {
                             msg += "\n        Last set from " + option.setFrom.path;
 
                         // FIXME: Message highlight group wrapping messes up the indent up for multi-arg verbose :set queries
-                        liberator.echo(<span highlight="CmdOutput">{msg}</span>);
+                        liberator.echo(xml`<span highlight="CmdOutput">${msg}</span>`);
                     }
                 }
                 // write access
@@ -1099,21 +1099,23 @@ const Options = Module("options", {
                 args = args.string;
 
                 if (!args) {
+                    var hasRow = false;
                     let str =
-                        <table>
-                        {
-                            template.map(liberator.globalVariables, function ([i, value]) {
+                        xml`<table>
+                        ${
+                            template.map2(xml, liberator.globalVariables, function ([i, value]) {
                                 let prefix = typeof value == "number"   ? "#" :
                                              typeof value == "function" ? "*" :
                                                                           " ";
-                                return <tr>
-                                            <td style="width: 200px;">{i}</td>
-                                            <td>{prefix}{value}</td>
-                                       </tr>;
+                                if (!hasRow) hasRow = true;
+                                return xml`<tr>
+                                            <td style="width: 200px;">${i}</td>
+                                            <td>${prefix}${value}</td>
+                                       </tr>`;
                             })
                         }
-                        </table>;
-                    if (str.*.length())
+                        </table>`;
+                    if (hasRow)
                         liberator.echo(str, commandline.FORCE_MULTILINE);
                     else
                         liberator.echomsg("No variables found");
