@@ -19,7 +19,7 @@
  *            | groupname css-selector
  */
 // <css>
-Highlights.prototype.CSS = <><![CDATA[
+Highlights.prototype.CSS = `
     Boolean     color: red;
     Function    color: navy;
     Null        color: blue;
@@ -46,14 +46,14 @@ Highlights.prototype.CSS = <><![CDATA[
     PromptText      color: white; background: purple;
 
     CmdOutput             white-space: pre;
-    CmdLine               background: white; color: black; -moz-transition: all 0.25s;
+    CmdLine               background: white; color: black; transition: all 0.25s;
     CmdLine>*             font-family: monospace;
 
     ContentSeparator      border-top: 1px dotted gray; display: -moz-box;
 
     CompGroup
     CompGroup:not(:first-of-type)  margin-top: 1ex;
-    CompTitle            font-weight: bold; background: -moz-linear-gradient(19% 75% 90deg, #DBDBDB, #D9D9D9, #E7E7E7 100%);
+    CompTitle            font-weight: bold; background: linear-gradient(to top, #DBDBDB 19%, #D9D9D9, #E7E7E7 100%);
     CompTitle>*          color: #333; border-top: 1px solid gray; border-bottom: 1px solid #BBB; padding: 1px 0.5ex; text-shadow: 1px 1px 0px #E0E0E0;
     CompMsg              font-style: italic; margin-left: 16px;
     CompItem
@@ -205,14 +205,7 @@ Highlights.prototype.CSS = <><![CDATA[
     HelpWarning                                 color: red; font-weight: bold;
 
     Logo
-
-    Search,,* {
-        font-size: inherit;
-        padding: 0;
-        color: black;
-        background-color: yellow;
-    }
-    ]]></>.toString();
+    `;
 
 /**
  * A class to manage highlighting rules. The parameters are the
@@ -241,7 +234,7 @@ function Highlights(name, store) {
 
     function keys() [k for ([k, v] in Iterator(highlight))].sort();
 
-    this.__iterator__ = function () (highlight[v] for ([k, v] in Iterator(keys())));
+    this.__iterator__ = function () (highlight[v] for (v of keys()));
 
     this.get = function (k) highlight[k];
     this.set = function (key, newStyle, force, append) {
@@ -501,7 +494,7 @@ function Styles(name, store) {
         if (matches.length == 0)
             return null;
 
-        for (let [, sheet] in Iterator(matches.reverse())) {
+        for (let sheet of matches.reverse()) {
             sheet.enabled = false;
             if (name)
                 delete names[name];
@@ -567,7 +560,7 @@ Module("styles", {
                     catch (e) {}
                     context.fork("others", 0, this, function (context) {
                         context.title = ["Site"];
-                        context.completions = [[s, ""] for ([, s] in Iterator(styles.sites))];
+                        context.completions = [[s, ""] for (s of styles.sites)];
                     });
                 }
             });
@@ -591,7 +584,7 @@ Module("styles", {
                           key,
                           sheet.sites.join(","),
                           sheet.css]
-                         for ([i, [key, sheet]] in Iterator(list))
+                         for ([key, sheet] of list)
                              if ((!filter || sheet.sites.indexOf(filter) >= 0) && (!name || sheet.name == name))));
 
                     commandline.echo(str, commandline.HL_NORMAL, commandline.FORCE_MULTILINE);
@@ -671,7 +664,7 @@ Module("styles", {
                 options: [[["-index", "-i"], commands.OPTION_INT, null,
                             function (context) {
                                 context.compare = CompletionContext.Sort.number;
-                                return [[i, <>{sheet.sites.join(",")}: {sheet.css.replace("\n", "\\n")}</>]
+                                return [[i, `${sheet.sites.join(",")}: ${sheet.css.replace("\n", "\\n")}`]
                                         for ([i, sheet] in styles.userSheets)
                                         if (!cmd.filter || cmd.filter(sheet))];
                             }],
@@ -735,7 +728,7 @@ Module("highlight", {
         commands.add(["hi[ghlight]"],
             "Set the style of certain display elements",
             function (args) {
-                let style = <><![CDATA[
+                let style = `
                     ;
                     display: inline-block !important;
                     position: static !important;
@@ -743,7 +736,7 @@ Module("highlight", {
                     width: 3em !important; min-width: 3em !important; max-width: 3em !important;
                     height: 1em !important; min-height: 1em !important; max-height: 1em !important;
                     overflow: hidden !important;
-                ]]></>;
+                `;
                 let clear = args[0] == "clear";
                 if (clear)
                     args.shift();
@@ -755,8 +748,8 @@ Module("highlight", {
                     // List matching keys
                     let str = template.tabular(["Key", { header: "Sample", style: "text-align: center" }, "CSS"],
                         ([h.class,
-                          <span style={h.value + style}>XXX</span>,
-                          template.highlightRegexp(h.value, /\b[-\w]+(?=:)/g, function (str) <span style="font-weight: bold;">{str}</span>)]
+                          xml`<span style=${h.value + style}>XXX</span>`,
+                          template.highlightRegexp(h.value, /\b[-\w]+(?=:)/g, function (str) xml`<span style="font-weight: bold;">${str}</span>`)]
                             for (h in highlight)
                                 if (!key || h.class.indexOf(key) > -1)));
 
