@@ -5,10 +5,22 @@
 
 /** @scope modules */
 
-const XHTML = Namespace("html", "http://www.w3.org/1999/xhtml");
-const XUL = Namespace("xul", "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
-const NS = Namespace("liberator", "http://vimperator.org/namespaces/liberator");
-default xml namespace = XHTML;
+//xxx:
+this.$$Namespace = function (prefix, uri) {
+    return "XMLList" in window ? Namespace(prefix, uri)
+        : {
+            prefix: prefix,
+            uri: uri,
+            toString: function () {
+                return this.uri;
+            },
+    };
+}
+const XHTML = $$Namespace("html", "http://www.w3.org/1999/xhtml");
+const XUL = $$Namespace("xul", "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
+const NS =  $$Namespace("liberator", "http://vimperator.org/namespaces/liberator");
+if ("XMLList" in window) eval("default xml namespace = XHTML");
+delete this.$$Namespace;
 
 const Util = Module("util", {
     init: function () {
@@ -91,66 +103,6 @@ const Util = Module("util", {
         return fixup.createFixupURI(str, fixup.FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP);
     },
 
-    /**
-     * Normalises URI escapes to provide a more aesthetic view, without changing
-     * the destination (leaves important chars, and invisible chars encoded)
-     *
-     * If conservative is set, then extra characters will be encoded to
-     * facilitate yanking and pasting of the result sanely.
-     *
-     * @param {string} url
-     * @param {boolean} conservative
-     * @returns {string}
-     */
-    // ripped from Firefox; modified
-    losslessDecodeURI: function losslessDecodeURI(url, conservative) {
-        // Countermeasure for "Error: malformed URI sequence".
-        // This error occurs when URL is encoded by not UTF-8 encoding.
-        function _decodeURI(url) {
-            try {
-                return decodeURI(url);
-            }
-            catch (e) {
-                return url;
-            }
-        };
-
-        // 1. decodeURI decodes %25 to %, which creates unintended
-        //    encoding sequences.
-        url = url.split("%25").map(_decodeURI).join("%25");
-        // 2. Re-encode whitespace so that it doesn't get eaten away
-        //    by the location bar (bug 410726).
-        url = url.replace(/[\r\n\t]/g, encodeURIComponent);
-
-        // Encode invisible characters (soft hyphen, zero-width space, BOM,
-        // line and paragraph separator, word joiner, invisible times,
-        // invisible separator, object replacement character) (bug 452979)
-        url = url.replace(/[\v\x0c\x1c\x1d\x1e\x1f\u00ad\u200b\ufeff\u2028\u2029\u2060\u2062\u2063\ufffc]/g,
-            encodeURIComponent);
-
-        // Encode bidirectional formatting characters.
-        // (RFC 3987 sections 3.2 and 4.1 paragraph 6)
-        url = url.replace(/[\u200e\u200f\u202a\u202b\u202c\u202d\u202e]/g,
-            encodeURIComponent);
-
-        // A few extra encodings to make links more portable.
-        if (conservative) {
-            // Encode ny remaining space
-            url = url.replace(/\s/g, encodeURIComponent);
-
-            // Encode trailing punctuation (http://en.wiktionary.org/wiki/e.g.)
-            // XXX: Much more could be put here, if it's desirable.
-            url = url.replace(/[.,]$/,
-                function(x) '%' + x.charCodeAt(0).toString(16).toUpperCase());
-
-            // Encode ' and " to facialiate using the URL in shell context
-            url = url.replace(/['"]/g,
-                function(x) '%' + x.charCodeAt(0).toString(16).toUpperCase());
-        }
-         
-        return url;
-    },
-
     /**
      * Converts HTML special characters in <b>str</b> to the equivalent HTML
      * entities.
@@ -193,22 +145,6 @@ const Util = Module("util", {
         return delimiter + str.replace(/([\\'"])/g, "\\$1").replace("\n", "\\n", "g").replace("\t", "\\t", "g") + delimiter;
     },
 
-    extend: function extend(dest) {
-        Array.slice(arguments, 1).filter(util.identity).forEach(function (src) {
-            for (let [k, v] in Iterator(src)) {
-                let get = src.__lookupGetter__(k),
-                    set = src.__lookupSetter__(k);
-                if (!get && !set)
-                    dest[k] = v;
-                if (get)
-                    dest.__defineGetter__(k, get);
-                if (set)
-                    dest.__defineSetter__(k, set);
-            }
-        });
-        return dest;
-    },
-
     /**
      * Returns an XPath union expression constructed from the specified node
      * tests. An expression is built with node tests for both the null and
@@ -261,8 +197,11 @@ const Util = Module("util", {
         while (cont) {
             cont = false;
             str = str.replace(resep, function (match, before) {
-                results.push(before);
-                cont = true;
+                cont = false;
+                if (before) {
+                    results.push(before);
+                    cont = true;
+                }
                 return "";
             });
         }
@@ -366,7 +305,7 @@ const Util = Module("util", {
                             data.push(" ");
                             data.push(name);
                             data.push('="');
-                            data.push(<>{value}</>.toXMLString());
+                            data.push(xml`${value}`.toString());
                             data.push('"')
                         }
                         if (node.localName in empty)
@@ -374,13 +313,13 @@ const Util = Module("util", {
                         else {
                             data.push(">");
                             if (node instanceof HTMLHeadElement)
-                                data.push(<link rel="stylesheet" type="text/css" href="help.css"/>.toXMLString());
+                                data.push(`<link rel="stylesheet" type="text/css" href="help.css"/>`/*toXMLString()*/);
                             Array.map(node.childNodes, arguments.callee);
                             data.push("</"); data.push(node.localName); data.push(">");
                         }
                         break;
                     case Node.TEXT_NODE:
-                        data.push(<>{node.textContent}</>.toXMLString());
+                        data.push(xml`${node.textContent}`.toString());
                 }
             }
             fix(content.document.documentElement);
@@ -467,13 +406,11 @@ const Util = Module("util", {
             null
         );
 
+        result.__iterator__ = asIterator
+                            ? function () { let elem; while ((elem = this.iterateNext())) yield elem; }
+                            : function () { for (let i = 0; i < this.snapshotLength; i++) yield this.snapshotItem(i); };
 
-        return {
-            __proto__: result,
-            __iterator__: asIterator
-                ? function () { let elem; while ((elem = this.iterateNext())) yield elem; }
-                : function () { for (let i = 0; i < this.snapshotLength; i++) yield this.snapshotItem(i); }
-        };
+        return result;
     },
 
     /**
@@ -552,13 +489,6 @@ const Util = Module("util", {
      * @returns {string}
      */
     objectToString: function objectToString(object, color) {
-        // Use E4X literals so html is automatically quoted
-        // only when it's asked for. No one wants to see &lt;
-        // on their console or :map :foo in their buffer
-        // when they expect :map <C-f> :foo.
-        XML.prettyPrinting = false;
-        XML.ignoreWhitespace = false;
-
         if (object === null)
             return "null\n";
 
@@ -581,14 +511,11 @@ const Util = Module("util", {
                 return node.localName.toLowerCase();
             }
             try {
-                let tag = "<" + [namespaced(elem)].concat(
-                    [namespaced(a) + "=" +  template.highlight(a.value, true)
-                     for ([i, a] in util.Array.iteritems(elem.attributes))]).join(" ");
-
-                if (!elem.firstChild || /^\s*$/.test(elem.firstChild) && !elem.firstChild.nextSibling)
-                    tag += '/>';
-                else
-                    tag += '>...</' + namespaced(elem) + '>';
+                let tag = xml`${`<${namespaced(elem)} `}${
+                    template.map2(xml, elem.attributes,
+                        function (a) xml`${namespaced(a)}=${template.highlight(a.value, true)}`, " ")}${
+                    !elem.firstChild || /^\s*$/.test(elem.firstChild) && !elem.firstChild.nextSibling
+                        ? "/>" : `>...</${namespaced(elem)}>`}`;
                 return tag;
             }
             catch (e) {
@@ -602,8 +529,8 @@ const Util = Module("util", {
         catch (e) {
             obj = "[Object]";
         }
-        obj = template.highlightFilter(util.clip(obj, 150), "\n", !color ? function () "^J" : function () <span highlight="NonText">^J</span>);
-        let string = <><span highlight="Title Object">{obj}</span>::<br/>&#xa;</>;
+        obj = template.highlightFilter(util.clip(obj, 150), "\n", !color ? function () "^J" : function () xml`<span highlight="NonText">^J</span>`);
+        let string = xml`<span highlight="Title Object">${obj}</span>::<br/>&#xa;`;
 
         let keys = [];
         try { // window.content often does not want to be queried with "var i in object"
@@ -613,7 +540,7 @@ const Util = Module("util", {
                 hasValue = false;
             }
             for (let i in object) {
-                let value = <><![CDATA[<no value>]]></>;
+                let value = xml`<![CDATA[<no value>]]>`;
                 try {
                     value = object[i];
                 }
@@ -626,12 +553,12 @@ const Util = Module("util", {
                 }
 
                 value = template.highlight(value, true, 150);
-                let key = <span highlight="Key">{i}</span>;
+                let key = xml`<span highlight="Key">${i}</span>`;
                 if (!isNaN(i))
                     i = parseInt(i);
                 else if (/^[A-Z_]+$/.test(i))
                     i = "";
-                keys.push([i, <>{key}{noVal ? "" : <>: {value}</>}<br/>&#xa;</>]);
+                keys.push([i, xml`${key}${noVal ? "" : xml`: ${value}`}<br/>&#xa;`]);
             }
         }
         catch (e) {}
@@ -641,8 +568,8 @@ const Util = Module("util", {
                 return a[0] - b[0];
             return String.localeCompare(a[0], b[0]);
         }
-        string += template.map(keys.sort(compare), function (f) f[1]);
-        return color ? string : [s for each (s in string)].join("");
+        xml["+="](string, template.map2(xml, keys.sort(compare), function (f) f[1]));
+        return string;
     },
 
     /**
@@ -743,6 +670,11 @@ const Util = Module("util", {
             urls = [str];
 
         return urls.map(function (url) {
+            url = url.trim();
+
+            if (!url)
+                return "";
+
             if (url.substr(0, 5) != "file:") {
                 try {
                     // Try to find a matching file.
@@ -753,9 +685,6 @@ const Util = Module("util", {
                 catch (e) {}
             }
 
-            // strip each 'URL' - makes things simpler later on
-            url = url.replace(/^\s+|\s+$/, "");
-
             // Look for a valid protocol
             let proto = url.match(/^([-\w]+):/);
             if (proto && Cc["@mozilla.org/network/protocol;1?name=" + proto[1]])
@@ -764,7 +693,8 @@ const Util = Module("util", {
 
             // Ok, not a valid proto. If it looks like URL-ish (foo.com/bar),
             // let Gecko figure it out.
-            if (/[.\/]/.test(url) && !/^\.|\s/.test(url) || /^[\w-.]+:\d+(?:\/|$)/.test(url))
+            if (!/^(["']).*\1$/.test(url) && /[.\/]/.test(url) && !/^\.|\s/.test(url) ||
+                /^[\w-.]+:\d+(?:\/|$)/.test(url))
                 return url;
 
             // TODO: it would be clearer if the appropriate call to
@@ -780,20 +710,46 @@ const Util = Module("util", {
 
             // Hmm. No defsearch? Let the host app deal with it, then.
             return url;
-        });
+        }).filter(function(url, i) i === 0 || url);
     },
 
+    /**
+     * Converts an string, TemplateXML object or E4X literal to a DOM node.
+     *
+     * @param {String|TemplateXML|xml} node
+     * @param {Document} doc
+     * @param {Object} nodes If present, nodes with the "key" attribute are
+     *     stored here, keyed to the value thereof.
+     * @returns {Node|DocumentFragment}
+     */
+    xmlToDom: function xmlToDom(node, doc, nodes) {
+        if (typeof node === "xml")
+            return this.xmlToDomForE4X(node, doc, nodes);
+
+        var dom = this.xmlToDomForTemplate(node, doc, nodes);
+
+        //xxx: change highlight's namespace
+        const str = "highlight";
+        var attr,
+            list = (dom.nodeType === 11 ? dom : dom.parentNode).querySelectorAll("[highlight]");
+        for (let node of list) {
+            attr = node.getAttribute(str);
+            node.removeAttribute(str);
+            node.setAttributeNS(NS, str, attr);
+        }
+        return dom;
+    },
     /**
      * Converts an E4X XML literal to a DOM node.
      *
      * @param {Node} node
      * @param {Document} doc
-     * @param {Object} nodes If present, nodes with the "key" attribute are
-     *     stored here, keyed to the value thereof.
+     * @param {Object} nodes
      * @returns {Node}
+     * @deprecated
+     * @see util.xmlToDom
      */
-    xmlToDom: function xmlToDom(node, doc, nodes) {
-        XML.prettyPrinting = false;
+    xmlToDomForE4X: function xmlToDomForE4X(node, doc, nodes) {
         if (node.length() != 1) {
             let domnode = doc.createDocumentFragment();
             for each (let child in node)
@@ -802,20 +758,88 @@ const Util = Module("util", {
         }
         switch (node.nodeKind()) {
         case "text":
-            return doc.createTextNode(node);
+            return doc.createTextNode(String(node));
         case "element":
             let domnode = doc.createElementNS(node.namespace(), node.localName());
-            for each (let attr in node.@*)
+            for each (let attr in node.attributes())
                 domnode.setAttributeNS(attr.name() == "highlight" ? NS.uri : attr.namespace(), attr.name(), String(attr));
-            for each (let child in node.*)
+            for each (let child in node.children())
                 domnode.appendChild(arguments.callee(child, doc, nodes));
-            if (nodes && node.@key)
-                nodes[node.@key] = domnode;
+            if (nodes && node.attribute("key"))
+                nodes[node.attribute("key")] = domnode;
             return domnode;
         default:
             return null;
         }
-    }
+    },
+    /**
+     * Converts an string of TemplateXML object to a DOM node.
+     *
+     * @param {String|TemplateXML} node
+     * @param {Document} doc
+     * @param {Object} nodes
+     * @returns {Node|DocumentFragment}
+     * @see util.xmlToDom
+     */
+    xmlToDomForTemplate: function xmlToDomForTemplate(node, doc, nodes) {
+        var range = doc.createRange();
+        var fragment = range.createContextualFragment(
+            xml`<div xmlns:ns=${NS} xmlns:xul=${XUL} xmlns=${XHTML}>${node}</div>`.toString());
+
+        range.selectNodeContents(fragment.firstChild);
+        var dom = range.extractContents();
+
+        range.detach();
+
+        if (nodes) {
+            for (let elm of dom.querySelectorAll("[key]"))
+                nodes[elm.getAttribute("key")] = elm;
+        }
+        return dom.childNodes.length === 1 ? dom.childNodes[0] : dom;
+    },
+    /**
+     * encoding dom
+     *
+     * @param {Node|Range|Selection|Document} node
+     * @param {String} type     example "text/plain", "text/html", "text/xml", "application/xhtml+xml" etc...
+     * @param {Number} flags    nsIDocumentEncoder.OutputXXXX
+     * @returns {String}
+     */
+    domToStr: let (Encoder = Components.Constructor("@mozilla.org/layout/documentEncoder;1?type=text/plain", "nsIDocumentEncoder", "init"))
+    function domToStr(node, type, flags) {
+        var doc, method;
+
+        if (node instanceof Document) {
+            doc = node;
+            node = null;
+            method = "setNode";
+        } else if (node instanceof Node) {
+            doc = node.ownerDocument;
+            method = "setNode";
+        } else if (node instanceof Range) {
+            doc = node.startContainer;
+            if (doc.ownerDocument) {
+                doc = doc.ownerDocument;
+            }
+            method = "setRange";
+        } else if (node instanceof Selection) {
+            // can not found document
+            if (node.rangeCount === 0) {
+                return "";
+            }
+            doc = node.getRangeAt(0).startContainer;
+            if (doc.ownerDocument) {
+                doc = doc.ownerDocument;
+            }
+            method = "setSelection";
+        } else {
+            return null;
+        }
+
+        var encoder = new Encoder(doc, type || "text/html", flags || 0);
+        encoder[method](node);
+        return encoder.encodeToString();
+    },
 }, {
     // TODO: Why don't we just push all util.BuiltinType up into modules? --djk
     /**
@@ -911,22 +935,7 @@ const Util = Module("util", {
          * @param {boolean} unsorted
          * @returns {Array}
          */
-        uniq: function uniq(ary, unsorted) {
-            let ret = [];
-            if (unsorted) {
-                for (let [, item] in Iterator(ary))
-                    if (ret.indexOf(item) == -1)
-                        ret.push(item);
-            }
-            else {
-                for (let [, item] in Iterator(ary.sort())) {
-                    if (item != last || !ret.length)
-                        ret.push(item);
-                    var last = item;
-                }
-            }
-            return ret;
-        }
+        uniq: function uniq(ary, unsorted) [...new Set(unsorted ? ary : ary.sort())],
     })
 });
 
